//package org.processmining.plugins.petrinet.importing.pnml;
//
//import java.io.InputStream;
//import java.util.ArrayList;
//import java.util.Collection;
//import java.util.HashMap;
//import java.util.Map;
//
//import org.processmining.contexts.gui.annotations.GUIAction;
//import org.processmining.contexts.uitopia.annotations.UIImportPlugin;
//import org.processmining.framework.abstractplugins.AbstractImportPlugin;
//import org.processmining.framework.plugin.PluginContext;
//import org.processmining.framework.plugin.annotations.Plugin;
//import org.processmining.models.connections.petrinets.behavioral.MarkingNetConnection;
//import org.processmining.models.graphbased.directed.petrinet.InhibitorNet;
//import org.processmining.models.graphbased.directed.petrinet.Petrinet;
//import org.processmining.models.graphbased.directed.petrinet.ResetInhibitorNet;
//import org.processmining.models.graphbased.directed.petrinet.ResetNet;
//import org.processmining.models.graphbased.directed.petrinet.elements.Place;
//import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
//import org.processmining.models.graphbased.directed.petrinet.impl.PetrinetFactory;
//import org.processmining.models.semantics.petrinet.Marking;
//import org.xmlpull.v1.XmlPullParser;
//import org.xmlpull.v1.XmlPullParserFactory;
//
///**
// * PnmlImport: Imports a ResetInhibitorNet from a PNML file. If the file
// * contains multiple nets, one of them is impo rted.
// * 
// * @author hverbeek
// * 
// */
//@Plugin(name = "Open PNML file", parameterLabels = { "Filename" }, returnLabels = { "Reset-Inhibitor Net", "Reset Net",
//		"Inhibitor Net", "Petri Net", "Reset-Inhibitor Marking", "Reset Marking", "Inhibitor Marking",
//		"PetriNet Marking" }, returnTypes = { ResetInhibitorNet.class, ResetNet.class, InhibitorNet.class,
//		Petrinet.class, Marking.class, Marking.class, Marking.class, Marking.class })
//@GUIAction(menuGroup = GUIAction.PETRINETGROUP, menuName = GUIAction.FILEMENU, showInToolbar = true, showInMenu = true, htmlToolTip = "<html>Open a PNML file, containing a Reset Inhibitor net.</html>")
//@UIImportPlugin(description = "EPNML files", extensions = { "pnml" })
//public class PnmlImport extends AbstractImportPlugin {
//
//	/**
//	 * Indices of objects in returned array of objects.
//	 */
//	public static final int RINET = 0; // Reset/Inhibitor net
//	public static final int RNET = 1; // Reset net
//	public static final int INET = 2; // Inhibitor net
//	public static final int PNET = 3; // Petri net
//	public static final int MAX = 4; // + MAX yields the marking of the net.
//
//	/**
//	 * These private classes store relevant information for nets, places,
//	 * transitions, and arcs.
//	 * 
//	 * There is no guarantee that every arc refers to nodes that were read
//	 * earlier in the PNML file. Therefore, we just gather the relevant
//	 * information, and if an entire net is read, we first create all places and
//	 * transitions, followed by the creation of all arcs.
//	 * 
//	 * @author hverbeek
//	 * 
//	 */
//	private class MyNet {
//		String name;
//
//		public MyNet() {
//			name = null;
//		}
//	}
//
//	private class MyPlace {
//		String id;
//		String name;
//		String marking;
//
//		public MyPlace() {
//			id = null;
//			name = null;
//			marking = null;
//		}
//	}
//
//	private class MyTransition {
//		String id;
//		String name;
//
//		public MyTransition() {
//			id = null;
//			name = null;
//		}
//	}
//
//	private class MyArc {
//		String target;
//		String source;
//		String type;
//		String weight;
//
//		public MyArc() {
//			target = null;
//			source = null;
//			type = null;
//			weight = null;
//		}
//	}
//
//	/**
//	 * Returns an object of 8 objects. The first four objects are nets, the
//	 * latter four are markings. The first object is a Reset-inhibitor net, the
//	 * fifth object is its marking. The second object is a Reset net, the sixth
//	 * object is its marking. The third object is an Inhibitor net, the seventh
//	 * object is its marking. The fourth object is a (plain) Petri net, the
//	 * eighth object is its marking.
//	 */
//	protected Object[] importFromStream(PluginContext context, InputStream input, String filename, long fileSizeInBytes)
//			throws Exception {
//		/**
//		 * Get an XML parser.
//		 */
//		XmlPullParserFactory factory = XmlPullParserFactory.newInstance();
//		factory.setNamespaceAware(true);
//		XmlPullParser xpp = factory.newPullParser();
//		/**
//		 * Initialize the parser.
//		 */
//		xpp.setInput(input, null);
//		/**
//		 * Read the PNML file.
//		 */
//		Collection<Object[]> nets = importPnml(xpp);
//		/**
//		 * Possibly, the PNML file contains a number of nets. For the time
//		 * being, get one of them.
//		 */
//		Object objects[] = (nets.isEmpty() ? null : nets.iterator().next());
//		if (objects != null) {
//			ResetInhibitorNet riNet = (ResetInhibitorNet) objects[RINET];
//			ResetNet rNet = (ResetNet) objects[RNET];
//			InhibitorNet iNet = (InhibitorNet) objects[INET];
//			Petrinet pNet = (Petrinet) objects[PNET];
//
//			Marking riMarking = (Marking) objects[MAX + RINET];
//			Marking rMarking = (Marking) objects[MAX + RNET];
//			Marking iMarking = (Marking) objects[MAX + INET];
//			Marking pMarking = (Marking) objects[MAX + PNET];
//
//			context.addConnection(new MarkingNetConnection(riNet, riMarking));
//			context.addConnection(new MarkingNetConnection(rNet, rMarking));
//			context.addConnection(new MarkingNetConnection(iNet, iMarking));
//			context.addConnection(new MarkingNetConnection(pNet, pMarking));
//
//			/**
//			 * Set the label for the nets in the object pool.
//			 */
//			context.getFutureResult(RINET).setLabel(riNet.getLabel());
//			context.getFutureResult(RNET).setLabel(rNet.getLabel());
//			context.getFutureResult(INET).setLabel(iNet.getLabel());
//			context.getFutureResult(PNET).setLabel(pNet.getLabel());
//
//			/**
//			 * And also for the markings.
//			 */
//			context.getFutureResult(MAX + RINET).setLabel("Marking of " + riNet.getLabel());
//			context.getFutureResult(MAX + RNET).setLabel("Marking of " + rNet.getLabel());
//			context.getFutureResult(MAX + INET).setLabel("Marking of " + iNet.getLabel());
//			context.getFutureResult(MAX + PNET).setLabel("Marking of " + pNet.getLabel());
//		}
//		/**
//		 * Return the nets and the markings.
//		 */
//		return objects;
//	}
//
//	/**
//	 * Imports a collection of ResetInhibitorNets using the given parser,
//	 * 
//	 * @param xpp
//	 *            The given parser.
//	 * @return The collection of nets retrieved from the given parser.
//	 * @throws Exception
//	 */
//	private Collection<Object[]> importPnml(XmlPullParser xpp) throws Exception {
//		Collection<Object[]> nets = null;
//		/**
//		 * Current net, place, transition, and arc.
//		 */
//		MyNet myNet = null;
//		MyPlace myPlace = null;
//		MyTransition myTransition = null;
//		MyArc myArc = null;
//		/**
//		 * Collections of all places, transitions, and arcs in the current net.
//		 */
//		Collection<MyPlace> myPlaces = new ArrayList<MyPlace>();
//		Collection<MyTransition> myTransitions = new ArrayList<MyTransition>();
//		Collection<MyArc> myArcs = new ArrayList<MyArc>();
//
//		String buffer = null;
//
//		/**
//		 * Get going...
//		 */
//		int eventType = xpp.getEventType();
//		while (eventType != XmlPullParser.END_DOCUMENT) {
//			if (eventType == XmlPullParser.START_TAG) {
//				if (xpp.getName().equalsIgnoreCase(PnmlTags.PNML.getLabel())) {
//					/**
//					 * Found a <pnml> tag, expect to retrieve some nets.
//					 */
//					nets = new ArrayList<Object[]>();
//				} else if (xpp.getName().equalsIgnoreCase(PnmlTags.NET.getLabel())) {
//					/**
//					 * Found a <net> tag. Initialize the current net.
//					 */
//					myNet = new MyNet();
//				} else if (xpp.getName().equalsIgnoreCase(PnmlTags.PLACE.getLabel())) {
//					/**
//					 * Found a <place> tag. Initialize the current place and add
//					 * it to the place collection.
//					 */
//					myPlace = new MyPlace();
//					myPlace.id = xpp.getAttributeValue(null, PnmlAttributes.ID.getLabel());
//					myPlaces.add(myPlace);
//				} else if (xpp.getName().equalsIgnoreCase(PnmlTags.TRANSITION.getLabel())) {
//					/**
//					 * Found a <transition> tag. Initialize the current
//					 * transition and add it to the transition collection.
//					 */
//					myTransition = new MyTransition();
//					myTransition.id = xpp.getAttributeValue(null, PnmlAttributes.ID.getLabel());
//					myTransitions.add(myTransition);
//				} else if (xpp.getName().equalsIgnoreCase(PnmlTags.ARC.getLabel())) {
//					/**
//					 * Found an <arc> tag. Initialize the current arc and add it
//					 * to the arc collection.
//					 */
//					myArc = new MyArc();
//					myArc.target = xpp.getAttributeValue(null, PnmlAttributes.TARGET.getLabel());
//					myArc.source = xpp.getAttributeValue(null, PnmlAttributes.SOURCE.getLabel());
//					myArcs.add(myArc);
//				} else if (xpp.getName().equalsIgnoreCase(PnmlTags.NAME.getLabel())) {
//					/**
//					 * Found a <name> tag.
//					 */
//					buffer = "";
//				} else if (xpp.getName().equalsIgnoreCase(PnmlTags.TYPE.getLabel())) {
//					/**
//					 * Found a <type> tag.
//					 */
//					buffer = "";
//				} else if (xpp.getName().equalsIgnoreCase(PnmlTags.INSCRIPTION.getLabel())) {
//					/**
//					 * Found a <inscription> tag.
//					 */
//					buffer = "";
//				} else if (xpp.getName().equalsIgnoreCase(PnmlTags.INITIALMARKING.getLabel())) {
//					/**
//					 * Found a <initialMarking> tag.
//					 */
//					buffer = "";
//				}
//			} else if (eventType == XmlPullParser.END_TAG) {
//				if (xpp.getName().equalsIgnoreCase(PnmlTags.PNML.getLabel())) {
//					/**
//					 * Found a </pnml> tag. Nothing to do.
//					 */
//				} else if (xpp.getName().equalsIgnoreCase(PnmlTags.NET.getLabel())) {
//					/**
//					 * Found a </net> tag. Create the net, and add it to the net
//					 * collection.
//					 */
//					if (myNet != null) {
//						nets.add(makeResetInhibitorNet(myNet, myPlaces, myTransitions, myArcs));
//					}
//					/**
//					 * Initialize the current net, places, transitions, and
//					 * arcs.
//					 */
//					myNet = null;
//					myPlaces = new ArrayList<MyPlace>();
//					myTransitions = new ArrayList<MyTransition>();
//					myArcs = new ArrayList<MyArc>();
//				} else if (xpp.getName().equalsIgnoreCase(PnmlTags.PLACE.getLabel())) {
//					/**
//					 * Found a </place> tag. No current place anymore.
//					 */
//					myPlace = null;
//				} else if (xpp.getName().equalsIgnoreCase(PnmlTags.TRANSITION.getLabel())) {
//					/**
//					 * Found a </transition> tag. No current transition anymore.
//					 */
//					myTransition = null;
//				} else if (xpp.getName().equalsIgnoreCase(PnmlTags.ARC.getLabel())) {
//					/**
//					 * Found an </arc> tag. No current arc anymore.
//					 */
//					myArc = null;
//				} else if (xpp.getName().equalsIgnoreCase(PnmlTags.NAME.getLabel())) {
//					/**
//					 * Found a </name> tag.
//					 */
//					if (myPlace != null) {
//						myPlace.name = buffer;
//					} else if (myTransition != null) {
//						myTransition.name = buffer;
//					}
//					buffer = null;
//				} else if (xpp.getName().equalsIgnoreCase(PnmlTags.TYPE.getLabel())) {
//					/**
//					 * Found a </type> tag.
//					 */
//					if (myArc != null) {
//						myArc.type = buffer;
//					}
//					buffer = null;
//				} else if (xpp.getName().equalsIgnoreCase(PnmlTags.INSCRIPTION.getLabel())) {
//					/**
//					 * Found a </inscription> tag.
//					 */
//					if (myArc != null) {
//						myArc.weight = buffer;
//					}
//					buffer = null;
//				} else if (xpp.getName().equalsIgnoreCase(PnmlTags.INITIALMARKING.getLabel())) {
//					/**
//					 * Found a </initialMarking> tag.
//					 */
//					if (myPlace != null) {
//						myPlace.marking = buffer;
//						buffer = "";
//					}
//					buffer = null;
//				}
//			} else if (eventType == XmlPullParser.TEXT) {
//				/**
//				 * Found a text element. Store it depending on the current
//				 * object.
//				 */
//				if (buffer != null) {
//					buffer = (buffer + xpp.getText()).trim();
//				}
//			}
//			eventType = xpp.next();
//		}
//		return nets;
//	}
//
//	/**
//	 * Creates four different nets from the information retrieved from the PNML
//	 * file.
//	 * 
//	 * @param myNet
//	 *            Relevant information retrieved for the net.
//	 * @param myPlaces
//	 *            Relevant information retrieved for the places.
//	 * @param myTransitions
//	 *            Relevant information retrieved for the transitions.
//	 * @param myArcs
//	 *            Relevant information retrieved for the arcs.
//	 * @return The created nets: ResetInhibitorNet, ResetNet, InhibitorNet,
//	 *         Petrinet.
//	 */
//	private Object[] makeResetInhibitorNet(MyNet myNet, Collection<MyPlace> myPlaces,
//			Collection<MyTransition> myTransitions, Collection<MyArc> myArcs) {
//		/**
//		 * Create the nets.
//		 */
//		ResetInhibitorNet riNet = PetrinetFactory.newResetInhibitorNet(myNet.name != null ? myNet.name
//				: "PNML import (Reset/Inhibitor net)");
//		ResetNet rNet = PetrinetFactory.newResetNet(myNet.name != null ? myNet.name : "PNML import (Reset net)");
//		InhibitorNet iNet = PetrinetFactory.newInhibitorNet(myNet.name != null ? myNet.name
//				: "PNML import (Inhibitor net)");
//		Petrinet pNet = PetrinetFactory.newPetrinet(myNet.name != null ? myNet.name : "PNML import (Petri net)");
//
//		Marking riMarking = new Marking();
//		Marking rMarking = new Marking();
//		Marking iMarking = new Marking();
//		Marking pMarking = new Marking();
//
//		/**
//		 * These mapping map all ids to nodes (places, transitions). Handy when
//		 * creating arcs, as every arc refers to the id of its target and
//		 * source.
//		 */
//		Map<String, Place> riPlaceMap = new HashMap<String, Place>();
//		Map<String, Place> rPlaceMap = new HashMap<String, Place>();
//		Map<String, Place> iPlaceMap = new HashMap<String, Place>();
//		Map<String, Place> pPlaceMap = new HashMap<String, Place>();
//		Map<String, Transition> riTransitionMap = new HashMap<String, Transition>();
//		Map<String, Transition> rTransitionMap = new HashMap<String, Transition>();
//		Map<String, Transition> iTransitionMap = new HashMap<String, Transition>();
//		Map<String, Transition> pTransitionMap = new HashMap<String, Transition>();
//
//		/**
//		 * Create all places.
//		 */
//		for (MyPlace myPlace : myPlaces) {
//			String name = (myPlace.name != null ? myPlace.name : myPlace.id);
//			riPlaceMap.put(myPlace.id, riNet.addPlace(name));
//			rPlaceMap.put(myPlace.id, rNet.addPlace(name));
//			iPlaceMap.put(myPlace.id, iNet.addPlace(name));
//			pPlaceMap.put(myPlace.id, pNet.addPlace(name));
//			Integer tokens = 0;
//			if (myPlace.marking != null) {
//				try {
//					tokens = Integer.valueOf(myPlace.marking);
//				} catch (NumberFormatException e) {
//				}
//			}
//			if (tokens > 0) {
//				riMarking.add(riPlaceMap.get(myPlace.id), tokens);
//				rMarking.add(rPlaceMap.get(myPlace.id), tokens);
//				iMarking.add(iPlaceMap.get(myPlace.id), tokens);
//				pMarking.add(pPlaceMap.get(myPlace.id), tokens);
//			}
//		}
//		/**
//		 * Create all transitions.
//		 */
//		for (MyTransition myTransition : myTransitions) {
//			String name = (myTransition.name != null ? myTransition.name : myTransition.id);
//			riTransitionMap.put(myTransition.id, riNet.addTransition(name));
//			rTransitionMap.put(myTransition.id, rNet.addTransition(name));
//			iTransitionMap.put(myTransition.id, iNet.addTransition(name));
//			pTransitionMap.put(myTransition.id, pNet.addTransition(name));
//			if (myTransition.name == null) {
//				/**
//				 * No name given, set it to be invisible.
//				 */
//				riTransitionMap.get(myTransition.id).setInvisible(true);
//				rTransitionMap.get(myTransition.id).setInvisible(true);
//				iTransitionMap.get(myTransition.id).setInvisible(true);
//				pTransitionMap.get(myTransition.id).setInvisible(true);
//			}
//		}
//		/**
//		 * Create all arcs. Note that all places and transitions should be there
//		 * now.
//		 */
//		for (MyArc myArc : myArcs) {
//			Integer weight = 1;
//			if (myArc.weight != null) {
//				try {
//					weight = Integer.valueOf(myArc.weight);
//				} catch (NumberFormatException e) {
//				}
//			}
//			if (riPlaceMap.containsKey(myArc.source) && riTransitionMap.containsKey(myArc.target)) {
//				if (myArc.type == null) {
//					riNet.addArc(riPlaceMap.get(myArc.source), riTransitionMap.get(myArc.target), weight);
//					rNet.addArc(rPlaceMap.get(myArc.source), rTransitionMap.get(myArc.target), weight);
//					iNet.addArc(iPlaceMap.get(myArc.source), iTransitionMap.get(myArc.target), weight);
//					pNet.addArc(pPlaceMap.get(myArc.source), pTransitionMap.get(myArc.target), weight);
//				} else if (myArc.type.equalsIgnoreCase(PnmlTexts.RESET.getLabel())) {
//					riNet.addResetArc(riPlaceMap.get(myArc.source), riTransitionMap.get(myArc.target));
//					rNet.addResetArc(rPlaceMap.get(myArc.source), rTransitionMap.get(myArc.target));
//					/**
//					 * Discard reset arcs for InhibitorNet and Petrinet
//					 */
//				} else if (myArc.type.equalsIgnoreCase(PnmlTexts.INHIBITOR.getLabel())) {
//					riNet.addInhibitorArc(riPlaceMap.get(myArc.source), riTransitionMap.get(myArc.target));
//					iNet.addInhibitorArc(iPlaceMap.get(myArc.source), iTransitionMap.get(myArc.target));
//					/**
//					 * Discard inhibitor arcs for ResetNet and Petrinet
//					 */
//				}
//			} else if (riTransitionMap.containsKey(myArc.source) && riPlaceMap.containsKey(myArc.target)) {
//				if (myArc.type == null) {
//					riNet.addArc(riTransitionMap.get(myArc.source), riPlaceMap.get(myArc.target), weight);
//					rNet.addArc(rTransitionMap.get(myArc.source), rPlaceMap.get(myArc.target), weight);
//					iNet.addArc(iTransitionMap.get(myArc.source), iPlaceMap.get(myArc.target), weight);
//					pNet.addArc(pTransitionMap.get(myArc.source), pPlaceMap.get(myArc.target), weight);
//				} else if (myArc.type.equalsIgnoreCase(PnmlTexts.RESET.getLabel())) {
//					/**
//					 * Reset arc from transition to place. Reverse it.
//					 */
//					riNet.addResetArc(riPlaceMap.get(myArc.target), riTransitionMap.get(myArc.source));
//					rNet.addResetArc(rPlaceMap.get(myArc.target), rTransitionMap.get(myArc.source));
//				} else if (myArc.type.equalsIgnoreCase(PnmlTexts.INHIBITOR.getLabel())) {
//					/**
//					 * Inhibitor arc from transition to place. Reverse it.
//					 */
//					riNet.addInhibitorArc(riPlaceMap.get(myArc.target), riTransitionMap.get(myArc.source));
//					iNet.addInhibitorArc(iPlaceMap.get(myArc.target), iTransitionMap.get(myArc.source));
//				}
//			}
//		}
//		/**
//		 * Return the nets and the markings.
//		 */
//		Object[] objects = new Object[2 * MAX];
//		objects[RINET] = riNet;
//		objects[RNET] = rNet;
//		objects[INET] = iNet;
//		objects[PNET] = pNet;
//		objects[MAX + RINET] = riMarking;
//		objects[MAX + RNET] = rMarking;
//		objects[MAX + INET] = iMarking;
//		objects[MAX + PNET] = pMarking;
//		return objects;
//	}
//
//}
