package org.processmining.plugins.petrinet.importing.pnml;

import java.util.HashMap;

/**
 * Encodes/decodes XML entities for the PNML export/import. 
 * The XmlPullParser also does the decoding, so the decoding is here for symmetry purposes only.
 * 
 * @author hverbeek
 *
 */
public class XmlEntities {
	private static HashMap<String, String> encodeMap;
	private static HashMap<String, String> decodeMap;
	
	final static String AMP = "&";
	final static String XML_AMP = "&amp;";
	
	/**
	 * Populate the maps.
	 */
	static {
		encodeMap = new HashMap<String, String>();
		decodeMap = new HashMap<String, String>();
		
		encodeMap.put("<", "&lt;");
		encodeMap.put(">", "&gt;");
		encodeMap.put("\"", "&quot;");
		encodeMap.put("'", "&apos;");
		
		decodeMap.put("&lt;", "<");
		decodeMap.put("&gt;", ">");
		decodeMap.put("&quot;", "\"");
		decodeMap.put("&apos;", "'");
	}

	/**
	 * Encodes an XML string (replaces & with &amp; etc.)
	 * @param s The string to encode.
	 * @return
	 */
	public static String encode(String s) {
		/**
		 * First replace '&' with '&amp;', as the other replacements will introduce '&'s.
		 */
		s = s.replace(AMP, XML_AMP);
		for (String key : encodeMap.keySet()) {
			s = s.replace(key, encodeMap.get(key));
		}
		return s;
	}

	/**
	 * Decodes an XML string (replaces &amp; with & etc.)
	 * @param s The string to decode.
	 * @return
	 */
	public static String decode(String s) {
		for (String key : decodeMap.keySet()) {
			s = s.replace(key, decodeMap.get(key));
		}
		/**
		 * Last replace '&amp;' with '&'.
		 */
		s = s.replace(XML_AMP, AMP);
		return s;
	}
}
