package org.processmining.plugins.testbed.experiment;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Vector;
import java.util.concurrent.BlockingQueue;

import org.apache.commons.math.stat.descriptive.moment.Mean;
import org.apache.commons.math.stat.descriptive.moment.StandardDeviation;
import org.apache.commons.math.stat.descriptive.rank.Percentile;
import org.processmining.plugins.testbed.result.Report;
import org.processmining.plugins.testbed.util.LabelledPnet;

/**
 * The class invokes the BFSPath calculator to compute the length of the shortest path of a Petri net
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *
 */
public class BFSPath extends Thread
{	
	private BlockingQueue<LabelledPnet> shortestLengthQueue; 
	private BlockingQueue<Vector<Report>> shortestPathReport;
	private int iterationNum;
	private int xthPercentile;
	

	
	public BFSPath(BlockingQueue<LabelledPnet> queue1, BlockingQueue<Vector<Report>> queue2, int iterationNum, int xthPercentile)
	{			
		this.shortestLengthQueue = queue1;
		this.shortestPathReport = queue2;
		this.iterationNum = iterationNum;
		this.xthPercentile = xthPercentile;

		start();
	}

	public void run()
	{		
		LabelledPnet labelledNet = null;
		double numOfTestNotBeyondBound = 0;
		double[] results = new double[iterationNum];
		Mean mean = new Mean();
		StandardDeviation stddev = new StandardDeviation(false);
		Percentile percentile = new Percentile();
		
		for (int i = 0; i < iterationNum; i++)
		{
			try { labelledNet = shortestLengthQueue.take(); } 
			catch (InterruptedException e) { e.printStackTrace(); }
			
			BFSPathCalculator n = new BFSPathCalculator(labelledNet.getPnet());
			results[i] = n.getShortestPath().getLength();
			numOfTestNotBeyondBound++;
		}

		
		NumberFormat  formatter  =  new DecimalFormat("#.00");
		Vector<Report> reports = new Vector<Report>();		
		reports.add(new Report("length of shortest path", "---",
				formatter.format(mean.evaluate(results, 0, results.length)), 
				formatter.format(stddev.evaluate(results, 0, results.length)), 
				formatter.format(percentile.evaluate(results, 0, results.length, 50)),
				formatter.format(percentile.evaluate(results, 0, results.length, xthPercentile)),
				formatter.format(percentile.evaluate(results, 0, results.length, 100-xthPercentile))));

		try { shortestPathReport.put(reports); } 
		catch (InterruptedException e) { e.printStackTrace(); }
	}
}