package org.processmining.plugins.testbed.experiment;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.testbed.result.PathResult;



/**
 * The class computes the length of a shortest path from the initial place to the final place of a workflow net
 * using bread first search algorithm
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *
 */
public class BFSPathCalculator 
{
	private Petrinet pnet; 
	private Place initialPlace;
	private Place finalPlace;
	private List<PetrinetNode> container;
	private boolean found;
	private ArrayList<PetrinetNode> shortestPath; 
	private List<ArrayList<PetrinetNode>> pathContainer; 
	private ArrayList<PetrinetNode> pathList;
	private PathResult pathResult;

	
	public BFSPathCalculator(Petrinet pnet)
	{
		this.pnet = pnet;
		this.initialPlace = null;
		this.finalPlace = null;
		this.container = new LinkedList<PetrinetNode>();
		this.found = false;
		this.shortestPath = new ArrayList<PetrinetNode>();
		this.pathContainer = new LinkedList<ArrayList<PetrinetNode>>();
		this.pathList = new ArrayList<PetrinetNode>();
		this.pathResult = new PathResult();
		
		getInitFinalPlace();
		search();
		calLength();
	}

	
	
	private void getInitFinalPlace()
	{
		boolean ifound = false;
		boolean ffound = false;
		Collection<Place> places = pnet.getPlaces();

		for (Place p : places)
		{
			if (pnet.getInEdges(p).size() == 0)
			{
				initialPlace = p;
				ifound = true;
			}
			if (pnet.getOutEdges(p).size() == 0)
			{
				finalPlace = p;
				ffound = true;
			}
			if (ifound && ffound)
			{
				break;
			}
		}
		
		container.add(initialPlace);
		pathList.add(initialPlace);
		pathContainer.add(pathList);
	}
	
	private void search()
	{
		do
		{
			PetrinetNode node = container.remove(0); 
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edges = pnet.getOutEdges(node);
			ArrayList<PetrinetNode> aPath = pathContainer.remove(0); 
			
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : edges)
			{
				PetrinetNode aNode = edge.getTarget();	
				ArrayList<PetrinetNode> bPath = new ArrayList<PetrinetNode>();
				bPath.addAll(aPath);
				bPath.add(aNode);
			
				if (aNode instanceof Place)
				{
					if ((Place)aNode == finalPlace)
					{
						found = true;
						shortestPath.addAll(bPath);
						break;
					}
				}
				container.add(aNode);
				pathContainer.add(bPath);
			}
		} while(!found);
	}
	
	private void calLength()
	{
		int l = 0;
		for (PetrinetNode n : shortestPath)
		{
			if (n instanceof Transition)
			{
				l++;
			}
		}
		pathResult.setLength(l);

	}
	
	/**
	 * returns the length of a shortest path of the given workflow net
	 * 
	 * @return pathResult, length of a shortest path
	 */
	public PathResult getShortestPath()
	{		
		return pathResult;
	}
}
