package org.processmining.plugins.testbed.experiment;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Vector;
import java.util.concurrent.BlockingQueue;

import org.apache.commons.math.stat.descriptive.moment.Mean;
import org.apache.commons.math.stat.descriptive.moment.StandardDeviation;
import org.apache.commons.math.stat.descriptive.rank.Percentile;
import org.processmining.plugins.testbed.result.Report;
import org.processmining.plugins.testbed.util.LabelledPnet;



/**
 * the class invokes the fanin/fanout calculator to compute the average fanin and fanout of nodes in a workflow net
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *
 */
public class FaninFanout extends Thread
{
	private BlockingQueue<LabelledPnet> faninFanoutQueue;
	private BlockingQueue<Vector<Report>> faninFanoutReport;
	private int iterationNum;
	private int xthPercentile;
	
	
	
	public FaninFanout(BlockingQueue<LabelledPnet> queue1, BlockingQueue<Vector<Report>> queue2, int iteraitonNum, int xthPercentile)
	{
		this.faninFanoutQueue = queue1;
		this.faninFanoutReport = queue2;
		this.iterationNum = iteraitonNum;
		this.xthPercentile = xthPercentile;
		
		start();
	}
	
	public void run()
	{
		LabelledPnet labelledNet = null;
		//int counter = 0;
		double numOfTestNotBeyondBound = 0;
		double[] placeFaninResults = new double[iterationNum];
		double[] placeFanoutResults = new double[iterationNum];
		double[] transFaninResults = new double[iterationNum];
		double[] transFanoutResults = new double[iterationNum];
		Mean mean = new Mean();
		StandardDeviation stddev = new StandardDeviation();
		Percentile percentile = new Percentile();
		
		
		for (int i = 0; i < iterationNum; i++)
		{
			try { labelledNet = faninFanoutQueue.take(); } 
			catch (InterruptedException e) { e.printStackTrace(); }
			
			FaninFanoutCalculator n = new FaninFanoutCalculator(labelledNet.getPnet());
			placeFaninResults[i] = n.getFaninFanout().getPlaceFanin();
			placeFanoutResults[i] = n.getFaninFanout().getPlaceFanout();
			transFaninResults[i] = n.getFaninFanout().getTransFanin();
			transFanoutResults[i] = n.getFaninFanout().getTransFanout();
			
			numOfTestNotBeyondBound++;
		}
		
		
		NumberFormat  formatter  =  new DecimalFormat("#.00");
		Vector<Report> reports = new Vector<Report>();
		reports.add(new Report("place fanin", "---", 
				formatter.format(mean.evaluate(placeFaninResults, 0, placeFaninResults.length)), 
				formatter.format(stddev.evaluate(placeFaninResults, 0, placeFaninResults.length)), 
				formatter.format(percentile.evaluate(placeFaninResults, 0, placeFaninResults.length, 50)),
				formatter.format(percentile.evaluate(placeFaninResults, 0, placeFaninResults.length, xthPercentile)),
				formatter.format(percentile.evaluate(placeFaninResults, 0, placeFaninResults.length, 100-xthPercentile))));
		reports.add(new Report("place fanout", "---", 
				formatter.format(mean.evaluate(placeFanoutResults, 0, placeFanoutResults.length)), 
				formatter.format(stddev.evaluate(placeFanoutResults, 0, placeFanoutResults.length)), 
				formatter.format(percentile.evaluate(placeFanoutResults, 0, placeFanoutResults.length, 50)),
				formatter.format(percentile.evaluate(placeFanoutResults, 0, placeFanoutResults.length, xthPercentile)),
				formatter.format(percentile.evaluate(placeFanoutResults, 0, placeFanoutResults.length, 100-xthPercentile))));
		reports.add(new Report("transition fanin", "---",  
				formatter.format(mean.evaluate(transFaninResults, 0, transFaninResults.length)), 
				formatter.format(stddev.evaluate(transFaninResults, 0, transFaninResults.length)), 
				formatter.format(percentile.evaluate(transFaninResults, 0, transFaninResults.length, 50)),
				formatter.format(percentile.evaluate(transFaninResults, 0, transFaninResults.length, xthPercentile)),
				formatter.format(percentile.evaluate(transFaninResults, 0, transFaninResults.length, 100-xthPercentile))));
		reports.add(new Report("transition fanout", "---",  
				formatter.format(mean.evaluate(transFanoutResults, 0, transFanoutResults.length)), 
				formatter.format(stddev.evaluate(transFanoutResults, 0, transFanoutResults.length)), 
				formatter.format(percentile.evaluate(transFanoutResults, 0, transFanoutResults.length, 50)),
				formatter.format(percentile.evaluate(transFanoutResults, 0, transFanoutResults.length, xthPercentile)),
				formatter.format(percentile.evaluate(transFanoutResults, 0, transFanoutResults.length, 100-xthPercentile))));
		
		try { faninFanoutReport.put(reports); } 
		catch (InterruptedException e) { e.printStackTrace(); }
	}
}