package org.processmining.plugins.testbed.experiment;

import java.util.Collection;

import org.apache.commons.math.stat.descriptive.moment.Mean;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.testbed.result.FaninFanoutResult;


/**
 * the class computes the average fanin and fanout of nodes in a workflow net
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *
 */
public class FaninFanoutCalculator 
{
	private Petrinet pnet;
	private FaninFanoutResult faninfanoutResult;
	private Mean placeFaninMean, placeFanoutMean, transFaninMean, transFanoutMean;
	
	
	public FaninFanoutCalculator(Petrinet pnet)
	{
		this.pnet = pnet;
		faninfanoutResult = new FaninFanoutResult();
		placeFaninMean = new Mean();
		placeFanoutMean = new Mean();
		transFaninMean = new Mean();
		transFanoutMean = new Mean();
		
		calFaninFanout();
	}
	
	
	
	private void calFaninFanout()
	{
		Collection<Place> places = pnet.getPlaces();
		Collection<Transition> transitions = pnet.getTransitions();
		
		for (Place p : places)
		{
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> inarcs = pnet.getInEdges(p);
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> outarcs = pnet.getOutEdges(p);
			placeFaninMean.increment(inarcs.size());
			placeFanoutMean.increment(outarcs.size());		
		}
		
		faninfanoutResult.setPlaceFanin(placeFaninMean.getResult());
		faninfanoutResult.setPlaceFanout(placeFanoutMean.getResult());

		for (Transition t : transitions)
		{
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> inarcs = pnet.getInEdges(t);
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> outarcs = pnet.getOutEdges(t);
			transFaninMean.increment(inarcs.size());
			transFanoutMean.increment(outarcs.size());
		}

		faninfanoutResult.setTransFanin(transFaninMean.getResult());
		faninfanoutResult.setTransFanout(transFanoutMean.getResult());			
	}
	
	
	
	/**
	 * returns the average fanin and fanout of nodes in a given workflow net
	 * 
	 * @return faninfanoutResult, the average fanin and fanout result
	 */
	public FaninFanoutResult getFaninFanout()
	{	
		return faninfanoutResult;
	}
}
