package org.processmining.plugins.testbed.experiment;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Vector;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import org.apache.commons.math.stat.descriptive.moment.Mean;
import org.apache.commons.math.stat.descriptive.moment.StandardDeviation;
import org.apache.commons.math.stat.descriptive.rank.Percentile;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.plugins.testbed.result.FindingPairResult;
import org.processmining.plugins.testbed.result.Report;
import org.processmining.plugins.testbed.util.LabelledPnet;


/**
 * The class invokes the causal pair calculator to compute how many traces and transitions to fire to find 
 * all the causal pairs in a workflow net
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *
 */
public class FindingPair extends Thread
{
	private int testNumPerNet;
	private int iterationNum;
	private PluginContext context;
	private int xthPercentile;
	private BlockingQueue<LabelledPnet> pairFindingQueue;
	private BlockingQueue<Vector<Report>> findingPairReport;
//	private int maxNumOfFiringPerTest;
//	private int maxNumOfFiringPerTrace;
//	private BlockingQueue<FindingPairResult> findingPairResult;

	
	public FindingPair(PluginContext context, BlockingQueue<LabelledPnet> queue1, BlockingQueue<Vector<Report>> queue2, int testNumPerNet, int iterationNum, int xthPercentile)
	{
		this.pairFindingQueue = queue1;
		this.findingPairReport = queue2;
		this.iterationNum = iterationNum;
		this.testNumPerNet = testNumPerNet;
		this.context = context;
//		this.maxNumOfFiringPerTest = bound;
//		this.maxNumOfFiringPerTrace = livelockBound;
		this.xthPercentile = xthPercentile;

		start();
	}
	

	
	public void run() 
	{
		
		LabelledPnet labelledPnet = null;
		//int counter = 0;
		//double numOfTestBeyondBound = 0;
		//double numOfTestNotBeyondBound = 0;
		Vector<Double> pathCoverageResults = new Vector<Double> ();
		Vector<Double> transCoverageResults = new Vector<Double>();
		Mean mean = new Mean();
		StandardDeviation stddev = new StandardDeviation(false);
		Percentile percentile = new Percentile();
		
		
		for (int i = 0; i < iterationNum; i++)
		{
			try {
				labelledPnet = pairFindingQueue.take();
				BlockingQueue<FindingPairResult> findingPairResult = new LinkedBlockingQueue<FindingPairResult>();
				
				
				
				if (new TotalLabelFinder(context, labelledPnet.getPnet()).findConnection())
				{
				
				
				int causalPairSize = new TotalLabelFinder(context, labelledPnet.getPnet()).getCausalPariNum();  //getAllCausalPair(labelledPnet.getPnet());
				for (int j = 0; j < this.testNumPerNet; j++)
				{
					new FindingPairPerTest((LabelledPnet)labelledPnet.clone(), findingPairResult, causalPairSize);//, maxNumOfFiringPerTest, maxNumOfFiringPerTrace);
				}
			
				for (int j = 0; j < testNumPerNet; j++)
				{
					FindingPairResult r = findingPairResult.take();
					
					//if (!r.getBeyondBound())
					//{	
						pathCoverageResults.add(r.getPathCoverage());
						transCoverageResults.add(r.getTransitionCoverage());
						//numOfTestNotBeyondBound++;
					//}
					//else
						//numOfTestBeyondBound++;
	      
				}
				}
				
			} catch (InterruptedException e) { e.printStackTrace(); }	
		}


		NumberFormat  formatter  =  new DecimalFormat("#.00");
		Vector<Report> reports = new Vector<Report>();		
		
		if (pathCoverageResults.size() == 0) { pathCoverageResults.add(0.00); }
		if (transCoverageResults.size() == 0) {transCoverageResults.add(0.00); }
		
		double[] pathCoverage = new double[pathCoverageResults.size()];
		for (int i = 0; i < pathCoverageResults.size(); i++) { pathCoverage[i] = pathCoverageResults.get(i); }
		double[] transCoverage = new double[transCoverageResults.size()];
		for (int i = 0; i < transCoverageResults.size(); i++) { transCoverage[i] = transCoverageResults.get(i); }
		
		reports.add(new Report("causal pair path coverage", "---", 
				formatter.format(mean.evaluate(pathCoverage, 0, pathCoverage.length)), 
				formatter.format(stddev.evaluate(pathCoverage, 0, pathCoverage.length)), 
				formatter.format(percentile.evaluate(pathCoverage, 0, pathCoverage.length, 50)),
				formatter.format(percentile.evaluate(pathCoverage, 0, pathCoverage.length, xthPercentile)),
				formatter.format(percentile.evaluate(pathCoverage, 0, pathCoverage.length, 100-xthPercentile))));

		reports.add(new Report("causal pair transition coverage", "---", 
				formatter.format(mean.evaluate(transCoverage, 0, transCoverage.length)), 
				formatter.format(stddev.evaluate(transCoverage, 0, transCoverage.length)), 
				formatter.format(percentile.evaluate(transCoverage, 0, transCoverage.length, 50)),
				formatter.format(percentile.evaluate(transCoverage, 0, transCoverage.length, xthPercentile)),
				formatter.format(percentile.evaluate(transCoverage, 0, transCoverage.length, 100-xthPercentile))));
		
		
		try { findingPairReport.put(reports); } 
		catch (InterruptedException e) { e.printStackTrace(); }	
	}

}