package org.processmining.plugins.testbed.experiment;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Random;
import java.util.Set;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.IllegalTransitionException;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.models.semantics.petrinet.PetrinetSemantics;
import org.processmining.models.semantics.petrinet.impl.PetrinetSemanticsFactory;
import org.processmining.plugins.testbed.result.FindingPairResult;
import org.processmining.plugins.testbed.util.LabelledPnet;
import org.processmining.plugins.testbed.util.Pair;

/**
 * The class computes how many traces and transitions to fire to
 * find all the causal pairs in a workflow net 
 *
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *
 */
public class FindingPairCalculator 
{
	private Petrinet pnet;
//	private ElementLabel transLabels;
	private PetrinetSemantics semantics;
	private Marking initialState;
	private double totalRun;	
	private boolean stop;
//	private int maxNumOfFiringPerTest;
//	private int maxzNumOfFiringPerTrace;
	private double totalFiring;
	private Set<Pair> testedPair;
	private int causalPairSize;
//	private boolean beyondBound;
	
	
	public FindingPairCalculator(LabelledPnet labelledPnet, int causalPairSize)//, int maxNumOfFiringPerTest, int maxNumOfFiringPerTrace)
	{
		this.pnet = labelledPnet.getPnet();		
		this.semantics = PetrinetSemanticsFactory.regularPetrinetSemantics(Petrinet.class);
		this.initialState = new Marking();
		this.totalRun = 0;
		this.stop = false;
		this.totalFiring = 0;
//      this.maxNumOfFiringPerTest = maxNumOfFiringPerTest;
//		this.maxzNumOfFiringPerTrace = maxNumOfFiringPerTrace;
		this.testedPair = new HashSet<Pair>();
		this.causalPairSize = causalPairSize;
//		this.beyondBound = false;
		
		
		getInitialPlace();
		
		while (!stop)
		{
			initSemantics();
			try {
				getFiringSequence();
			} 
			catch (IllegalTransitionException e) {
				e.printStackTrace();
			}	
		}
	}
	
	
	/**
	 * returns the number of traces and transitions to fire to find all the causal pairs in the workflow net given
	 * 
	 * @return r, number of traces and transitions to fire 
	 */
	public FindingPairResult getResult()
	{	
		FindingPairResult r = new FindingPairResult();
		r.setPathCoverage(totalRun);
		r.setTransitionCoverage(totalFiring);
		//r.setBeyondBound(beyondBound);

		return r;
	}
	
	
	private void getInitialPlace()
	{
		Collection<Place> places = pnet.getPlaces();
		for (Place p : places)
		{
			if (pnet.getInEdges(p).size() == 0)
			{
				initialState.add(p);
				break;
			}
		}
	}
	

	
	private void initSemantics()
	{	
		Collection<Transition> transitions = pnet.getTransitions();
		semantics.initialize(transitions, initialState); 
	}
	
	private void getFiringSequence() throws IllegalTransitionException
	{
		Iterator<Transition> iter = semantics.getExecutableTransitions().iterator();
		Object[] executables = null;
		totalRun++;
		Transition oldTrans = null;
	
		while (iter.hasNext())
		{	
			executables = semantics.getExecutableTransitions().toArray();
			int l = new Random().nextInt(executables.length);
			Transition t = (Transition)executables[l];
			semantics.executeExecutableTransition(t);
			iter = semantics.getExecutableTransitions().iterator(); 
			totalFiring++; 
			
//			if (totalFiring > maxNumOfFiringPerTest)
//			{
//				stop = true;
//				beyondBound = true;
//				return;
//			}
			
			if (oldTrans != null)
			{
				Pair pair = new Pair(oldTrans, t);
				testedPair.add(pair);
			}
			
			oldTrans = t;
		}

		if (testedPair.size() >= causalPairSize) { stop = true; } 
	}
}
