package org.processmining.plugins.testbed.experiment;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Vector;
import java.util.concurrent.BlockingQueue;

import org.apache.commons.math.stat.descriptive.moment.Mean;
import org.apache.commons.math.stat.descriptive.moment.StandardDeviation;
import org.apache.commons.math.stat.descriptive.rank.Percentile;
import org.processmining.plugins.testbed.result.Report;
import org.processmining.plugins.testbed.util.LabelledPnet;


/**
 * The class invokes node number calculator to compute number of nodes (places and transitions) in a Petri net
 *
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *
 */
public class NodeNum extends Thread
{
	private BlockingQueue<LabelledPnet> nodeNumQueue;
	private BlockingQueue<Vector<Report>> nodeNumReport;
	private int iterationNum;
	private int xthPercentile;
	

	
	public NodeNum(BlockingQueue<LabelledPnet> queue1, BlockingQueue<Vector<Report>> queue2, int iterationNum, int xthPercentile)
	{
		this.nodeNumQueue = queue1;
		this.nodeNumReport = queue2;
		this.iterationNum = iterationNum;
		this.xthPercentile = xthPercentile;
		
		start();
	}
	
	public void run()
	{
		LabelledPnet labelledNet = null;
		double[] placeNumResults = new double[iterationNum];
		double[] transNumResults = new double[iterationNum];
		Mean mean = new Mean();
		StandardDeviation stddev = new StandardDeviation(false);
		Percentile percentile = new Percentile();
		double numOfTestNotBeyondBound = 0;
		
		
		for (int i = 0; i < iterationNum; i++)
		{
			try { labelledNet = nodeNumQueue.take(); } 
			catch (InterruptedException e) { e.printStackTrace(); 
			}
			NodeNumCalculator n = new NodeNumCalculator(labelledNet.getPnet());
			placeNumResults[i] = n.getNodeNum().getPlaceNum();
			transNumResults[i] = n.getNodeNum().getTransNum();
			numOfTestNotBeyondBound++;
		}
		
		NumberFormat  formatter  =  new DecimalFormat("#.00");
		Vector<Report> reports = new Vector<Report>();
		reports.add(new Report("number of places", "---",
				formatter.format(mean.evaluate(placeNumResults, 0, placeNumResults.length)), 
				formatter.format(stddev.evaluate(placeNumResults, 0, placeNumResults.length)), 
				formatter.format(percentile.evaluate(placeNumResults, 0, placeNumResults.length, 50)),
				formatter.format(percentile.evaluate(placeNumResults, 0, placeNumResults.length, xthPercentile)),
				formatter.format(percentile.evaluate(placeNumResults, 0, placeNumResults.length, 100-xthPercentile))));
		reports.add(new Report("number of transitions", "---",
				formatter.format(mean.evaluate(transNumResults, 0, transNumResults.length)), 
				formatter.format(stddev.evaluate(transNumResults, 0, transNumResults.length)), 
				formatter.format(percentile.evaluate(transNumResults, 0, transNumResults.length, 50)),
				formatter.format(percentile.evaluate(transNumResults, 0, transNumResults.length, xthPercentile)),
				formatter.format(percentile.evaluate(transNumResults, 0, transNumResults.length, 100-xthPercentile))));
		
		try { nodeNumReport.put(reports); } 
		catch (InterruptedException e) { e.printStackTrace(); }	
	}
}
