package org.processmining.plugins.testbed.experiment;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Vector;
import java.util.concurrent.BlockingQueue;

import org.apache.commons.math.stat.descriptive.moment.Mean;
import org.apache.commons.math.stat.descriptive.moment.StandardDeviation;
import org.apache.commons.math.stat.descriptive.rank.Percentile;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.plugins.testbed.result.Report;
import org.processmining.plugins.testbed.util.LabelledPnet;




/**
 * the class detects whether or not a given net is sound using Woflan plugin
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *
 */
public class Soundness extends Thread
{
	private BlockingQueue<LabelledPnet> soundnessQueue;
	private BlockingQueue<Vector<Report>> soundnessReport;
	private int iterationNum;
	private PluginContext context;
	private int xthPercentile;
	
	//private int actualNum;

	
	
	public Soundness(BlockingQueue<LabelledPnet> pnetSoundness, BlockingQueue<Vector<Report>> soundnessReport, int iterationNum, PluginContext context, int xthPercentile)
	{
		this.soundnessQueue = pnetSoundness;
		this.soundnessReport = soundnessReport;
		this.iterationNum = iterationNum;
		this.context = context;
		this.xthPercentile = xthPercentile;
		
		//this.actualNum = 0;
		
		start();
	}

	
	public synchronized void run() 
	{
		LabelledPnet labelledNet = null;
		//double[] results = new double[iterationNum];
		
		Vector<Double> res = new Vector<Double>();
		
		Mean mean = new Mean();
		StandardDeviation stddev = new StandardDeviation(false);
		Percentile percentile = new Percentile();
		//double numOfTestNotBeyondBound = 0;
		
		
		
		for (int i = 0; i < iterationNum; i++)
		{
			try {
				labelledNet = soundnessQueue.take();
				
				
				
				//SoundnessCalculator s = new SoundnessCalculator(context.createChildContext(null), labelledNet.getPnet());
				
				SoundnessCalculator s = new SoundnessCalculator(context, labelledNet.getPnet());
				
				
				
				if (s.findConnection())
				{
				
				res.add(s.getSound());
				//actualNum++;
				
				//System.out.println(s.getSound());
				//System.out.println(actualNum);
				
				
				
				//results[i] = s.getSound();}
				
				
				//for (int j = 0; j < results.length; j++)
				//{
					//System.out.print(results[j] + "   ");
				//}
				
//				Woflan n = new Woflan();
//				String report;
//				report = n.diagnose(context, labelledNet.getPnet()).toString();
//				
//				
//				int sound = -1;
//				if (report.contains("sound"))
//					sound = 1;
//				else
//					sound = 0;
//				
//				results[i] = sound;
//				numOfTestNotBeyondBound++;
				
				
			}
			}
			catch (InterruptedException e) { e.printStackTrace(); }
			catch (Exception e) { e.printStackTrace(); }

		}
		
		
		double[] results = new double[res.size()];
		
		
		for (int j = 0; j < res.size(); j++)
		{
			results[j] = res.get(j);
		}
		
	
		NumberFormat  formatter  =  new DecimalFormat("#.00");
		Vector<Report> reports = new Vector<Report>();
		reports.add(new Report("soundness", "woflan",
				formatter.format(mean.evaluate(results, 0, results.length)), 
				formatter.format(stddev.evaluate(results, 0, results.length)), 
				formatter.format(percentile.evaluate(results, 0, results.length, 50)),
				formatter.format(percentile.evaluate(results, 0, results.length, xthPercentile)),
				formatter.format(percentile.evaluate(results, 0, results.length, 100-xthPercentile))));

		try { soundnessReport.put(reports); } 
		catch (InterruptedException e) { e.printStackTrace(); }
	}
}
