package org.processmining.plugins.testbed.experiment;

import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.plugins.petrinet.behavioralanalysis.woflan.Woflan;


/**
 * The class tests whether or not a workflow net is sound
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 15/June/2010
 *
 */
public class SoundnessCalculator 
{
	
	private Petrinet pnet;
	private double sound;
	private PluginContext context;
	private boolean connectionFound;
	
	
	
	public SoundnessCalculator(PluginContext context, Petrinet pnet)
	{
		this.pnet = pnet;
		this.context = context;
		this.sound = -1;
		this.connectionFound = false;
		
		test();
	}
	
	
	private void test()
	{
		Woflan n = new Woflan();
		String report = null;
		try {
			report = n.diagnose(context.createChildContext("Running Woflan on " + pnet.getLabel()), pnet).toString();
			if (report.contains("sound"))
				sound = 1;
			else
				sound = 0;
			
			connectionFound = true;
			
			//testing
			//System.out.println("sound is  "  +  sound);
		} 
		catch (Exception e) {
			connectionFound = false;
			
			//System.out.println("asdfasdfasdf");
			
			//e.printStackTrace();
		}
	}
	
	
	/**
	 * returns whether or not a workflow net is sound
	 * 
	 * @return sound, soundness of a workflow net
	 */
	public double getSound()
	{
		return sound;
	}
	
	
	public boolean findConnection()
	{
		return connectionFound;
	}

}

