package org.processmining.plugins.testbed.experiment;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import java.util.Stack;

import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.connections.petrinets.behavioral.AggTSGraphConnection;
import org.processmining.models.connections.petrinets.behavioral.StateSpaceConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.analysis.AggTSGraph;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.transitionsystem.ReachabilityGraph;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.models.semantics.petrinet.PetrinetSemantics;
import org.processmining.models.semantics.petrinet.impl.PetrinetSemanticsFactory;
import org.processmining.plugins.testbed.util.Pair;


/**
 * The class computes the total number of all the causal pairs in a workflow net using reachability graph 
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *
 */
public class TotalLabelFinder 
{
	private Petrinet pnet;
	private PluginContext context;
	private Set<Pair> causalPair;
	private boolean connectionFound; 
	
	
	
	public TotalLabelFinder(PluginContext context, Petrinet pnet)
	{
		this.pnet = pnet;
		this.context = context;
		this.causalPair = new HashSet<Pair>();
		this.connectionFound = false;
		
		calAllCausalPair();
	}
	
	private void calAllCausalPair()
	{
		Marking initialState = new Marking();
		PetrinetSemantics semantics = PetrinetSemanticsFactory.regularPetrinetSemantics(Petrinet.class);
		//Set<Pair> causalPair = new HashSet<Pair>();
		Set<org.processmining.models.graphbased.directed.transitionsystem.State> visitedState = new HashSet<org.processmining.models.graphbased.directed.transitionsystem.State>();
		
		
		Collection<Place> places = pnet.getPlaces();
		for (Place p : places)
		{
			if (pnet.getInEdges(p).size() == 0)
			{
				initialState.add(p);
				break;
			}
		}
		
		
		Stack<org.processmining.models.graphbased.directed.transitionsystem.State> tobeTestedState = new Stack<org.processmining.models.graphbased.directed.transitionsystem.State>();
		org.processmining.models.graphbased.directed.transitionsystem.State state = null;
		
		
		try {	
		
			//AggTSGraph aggGraph = context.tryToFindOrConstructFirstObject(AggTSGraph.class, AggTSGraphConnection.class, AggTSGraphConnection.AGGTSGRAPH, pnet, initialState, semantics);			
			//ReachabilityGraph covGraph = (ReachabilityGraph)aggGraph.get(AggTSGraph.REACHABILITYGRAPH);
			
			ReachabilityGraph covGraph = context.tryToFindOrConstructFirstObject(ReachabilityGraph.class, StateSpaceConnection.class, StateSpaceConnection.STATEPACE, pnet, initialState, semantics);
			tobeTestedState.push((org.processmining.models.graphbased.directed.transitionsystem.State) covGraph.getGraph().getNodes().toArray()[0]);
			
			
			connectionFound = true;
			
			
			while(tobeTestedState.size() != 0)
			{
				state = tobeTestedState.pop();
				visitedState.add(state);
				
				Collection<org.processmining.models.graphbased.directed.transitionsystem.Transition> outEdges = covGraph.getOutEdges(state);
				
				if (covGraph.getInEdges(state).size() != 0 && covGraph.getOutEdges(state).size() != 0)
				{
					Collection<org.processmining.models.graphbased.directed.transitionsystem.Transition> inEdges = covGraph.getInEdges(state);
					
					
					for (org.processmining.models.graphbased.directed.transitionsystem.Transition tIn : inEdges)
					{
						for (org.processmining.models.graphbased.directed.transitionsystem.Transition tOut : outEdges)
						{
							causalPair.add(new Pair(tIn, tOut));
						}
					}
					
				}
				
				
				for (org.processmining.models.graphbased.directed.transitionsystem.Transition t : outEdges)
				{
					if (!tobeTestedState.contains(t.getTarget()) && !visitedState.contains(t.getTarget()))
						tobeTestedState.push(t.getTarget());
				}
			}
		} catch (ConnectionCannotBeObtained e) {			
			connectionFound = false;
			//System.out.println("no connection found");
			//e.printStackTrace();
		}
		

//		catch (Exception e){
//			System.out.println("eror");
//		}
		
		//return causalPair.size();
	}
	
	
	/**
	 * returns the number of all the causal pairs in a workflow net
	 * 
	 * @return number of all the causal pairs
	 */
	public int getCausalPariNum()
	{
		return causalPair.size();
	}
	
	
	
	
	
	
	public boolean findConnection()
	{
		return connectionFound;
	}
	
	
}
