package org.processmining.plugins.testbed.generationrule;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;
import java.util.Set;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.testbed.util.ElementWeight;




/**
 * the class implements the Arc Bridge rule
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *
 */
public class ArcBridge extends GenerationRule
{
	private PetrinetNode sourceNode;
	private PetrinetNode targetNode;
	private Map<PetrinetNode, Integer> illegalSourceNode;
	private boolean changed2;

	public ArcBridge(Petrinet net, PetrinetNode initialNodeOfNet, PetrinetNode finalNodeOfNet, ElementWeight elementWeight)
	{
		super(net, initialNodeOfNet, finalNodeOfNet, elementWeight);
		this.sourceNode = null;
		this.targetNode = null;		
		this.illegalSourceNode = new HashMap<PetrinetNode, Integer>();
		this.changed2 = false;
		
		selectElement();
		
		if (!notFound)
		{
			addElementToPnet();
		}
	}
	
	
	private void disableNode()
	{
		for (PetrinetNode n : illegalSourceNode.keySet())
		{
			if (n instanceof Place)
			{
				elementWeight.changeElementWeight((Place)n, 0);
			}
			else
			{
				elementWeight.changeElementWeight((Transition)n, 0);
			}		
		}
	}
	
	private void enableNode()
	{
		for (PetrinetNode n : illegalSourceNode.keySet())
		{
			if (n instanceof Place)
			{
				elementWeight.changeElementWeight((Place)n, illegalSourceNode.get(n));
			}
			else
			{
				elementWeight.changeElementWeight((Transition)n, illegalSourceNode.get(n));
			}
		}
	}
	
	
	/**
	 * add an arc to bridge two selected nodes 
	 */
	protected void addElementToPnet()
	{
		if (sourceNode instanceof Place)
		{
			Arc r = pnet.addArc((Place)sourceNode, (Transition)targetNode);
			elementWeight.setWeight(r, elementWeight.addElementWeight(elementWeight.getElementWeight((Place)sourceNode), elementWeight.getElementWeight((Transition)targetNode)));
			elementWeight.changeElementWeight((Place)sourceNode);
			elementWeight.changeElementWeight((Transition)targetNode);
		}
		else
		{
			Arc r = pnet.addArc((Transition)sourceNode, (Place)targetNode);
			elementWeight.setWeight(r, elementWeight.addElementWeight(elementWeight.getElementWeight((Transition)sourceNode), elementWeight.getElementWeight((Place)targetNode)));
			elementWeight.changeElementWeight((Transition)sourceNode);
			elementWeight.changeElementWeight((Place)targetNode);
		}
	}
	
	
	/**
	 * selects two nodes to bridge using an arc 
	 */
	@SuppressWarnings("unchecked")
	protected void selectElement()
	{
		while(true)
		{		
			sourceNode = null;
			int weight1 = 0;
			boolean changed1 = false;
			
			if (finalNodeOfNet != null)
			{
				if (finalNodeOfNet instanceof Place)
				{
					weight1 = elementWeight.getElementWeight((Place)finalNodeOfNet);
					elementWeight.changeElementWeight((Place)finalNodeOfNet, 0);
					changed1 = true;
				}
				else
				{
					weight1 = elementWeight.getElementWeight((Transition)finalNodeOfNet);
					elementWeight.changeElementWeight((Transition)finalNodeOfNet, 0);
					changed1 = true;
				}
			}
	
			if (changed2)
			{
				disableNode();
			}
			
			int totalWeight = elementWeight.getPlaceTotalWeight() + elementWeight.getTransitionTotalWeight();//total of places and transitions
			
			if (totalWeight == 0)
			{
				notFound = true;
			
				if (changed1)
				{
					if (finalNodeOfNet instanceof Place)
					{
						elementWeight.changeElementWeight((Place)finalNodeOfNet, weight1);
					}
					else
					{
						elementWeight.changeElementWeight((Transition)finalNodeOfNet, weight1);
					}
				}
				
				if (changed2)
				{
					enableNode();
				}
				
				return;
			}
		
			int random = new Random().nextInt(totalWeight);
			int weightSoFar = 0;
			Map<Place, Integer> placeMap = elementWeight.getPlaceWeight();//get weights of all the transitions in the net
			Iterator<Place> it = placeMap.keySet().iterator();//to iterate all transitions in the key set

			while(it.hasNext())
			{
				Place p = it.next();
				weightSoFar += placeMap.get(p);
			
				if (weightSoFar > random)
				{
					sourceNode = p;
					break;
				}
			}
		
			if (sourceNode == null)
			{
				Map<Transition, Integer> transMap = elementWeight.getTransitionWeight();
				Iterator<Transition> it1 =  transMap.keySet().iterator();
			
				while(it1.hasNext())
				{
					Transition t = it1.next();
					weightSoFar += transMap.get(t);
				
					if (weightSoFar > random)
					{
						sourceNode = t;
						break;
					}
				}
			}
		
			if (changed1)
			{
				if (finalNodeOfNet instanceof Place)
				{
					elementWeight.changeElementWeight((Place)finalNodeOfNet, weight1);
				}
				else
				{
					elementWeight.changeElementWeight((Transition)finalNodeOfNet, weight1);
				}
			}
			
			if (changed2)
			{
				enableNode();
			}
		
			Map<PetrinetNode, Integer> illegalTargetNode = new HashMap<PetrinetNode, Integer>();
			Set<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> arcs = pnet.getEdges();
		
			if (initialNodeOfNet != null)
			{
				if (sourceNode instanceof Transition && initialNodeOfNet instanceof Place)
				{
					illegalTargetNode.put(initialNodeOfNet, elementWeight.getElementWeight((Place)initialNodeOfNet));
					elementWeight.changeElementWeight((Place)initialNodeOfNet, 0);
				}
				if (sourceNode instanceof Place && initialNodeOfNet instanceof Transition)
				{
					illegalTargetNode.put(initialNodeOfNet, elementWeight.getElementWeight((Transition)initialNodeOfNet));
					elementWeight.changeElementWeight((Transition)initialNodeOfNet, 0);
				}
			}
		
			for (PetrinetEdge arc : arcs)
			{
				if (arc.getSource().equals(sourceNode))
				{
					if (sourceNode instanceof Place)
					{
						illegalTargetNode.put((Transition)arc.getTarget(), elementWeight.getElementWeight((Transition)arc.getTarget()));
						elementWeight.changeElementWeight((Transition)arc.getTarget(), 0);
					}
					else
					{
						illegalTargetNode.put((Place)arc.getTarget(), elementWeight.getElementWeight((Place)arc.getTarget()));
						elementWeight.changeElementWeight((Place)arc.getTarget(), 0);
					}
				}
			} //also need to change the initial node to invalid as the target node 
		
		
			if (sourceNode instanceof Place)
			{
				totalWeight = elementWeight.getTransitionTotalWeight();
			
				if (totalWeight == 0)
				{
					for (PetrinetNode t : illegalTargetNode.keySet())
					{
						elementWeight.changeElementWeight((Transition)t, illegalTargetNode.get(t));
					}
				
					illegalSourceNode.put(sourceNode, elementWeight.getElementWeight((Place)sourceNode));
					changed2 = true;
				}
				else
				{
					Map<Transition, Integer> transMap = elementWeight.getTransitionWeight();//get weights of all the transitions in the net
					random = new Random().nextInt(totalWeight);
					weightSoFar = 0;
					Iterator<Transition> it1 = transMap.keySet().iterator();//to iterate all transitions in the key set

					while(it1.hasNext())
					{
						Transition t = it1.next();
						weightSoFar += transMap.get(t);
				
						if (weightSoFar > random)
						{
							targetNode = t;
							break;
						}
					}
				
					for (PetrinetNode t : illegalTargetNode.keySet())
					{
						elementWeight.changeElementWeight((Transition)t, illegalTargetNode.get(t));
					}
					
					break;
				}
			}
			else
			{
				totalWeight = elementWeight.getPlaceTotalWeight();
				
				if (totalWeight == 0)
				{
					for(PetrinetNode p : illegalTargetNode.keySet())
					{
						elementWeight.changeElementWeight((Place)p, illegalTargetNode.get(p));
					}
				
					illegalSourceNode.put(sourceNode, elementWeight.getElementWeight((Transition)sourceNode));
					changed2 = true;
				}
				else
				{
					placeMap = elementWeight.getPlaceWeight();//get weights of all the transitions in the net
					random = new Random().nextInt(totalWeight);
					weightSoFar = 0;
					it = placeMap.keySet().iterator();//to iterate all transitions in the key set

					while(it.hasNext())
					{
						Place p = it.next();
						weightSoFar += placeMap.get(p);
				
						if (weightSoFar > random)
						{
							targetNode = p;
							break;
						}
					}
				
					for(PetrinetNode p : illegalTargetNode.keySet())
					{
						elementWeight.changeElementWeight((Place)p, illegalTargetNode.get(p));
					}
					
					break;
				}
			}
		}
	}
}
