package org.processmining.plugins.testbed.generationrule;

import java.util.Iterator;
import java.util.Map;
import java.util.Random;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.testbed.util.ElementWeight;



/**
 * the class implements the Arc Refinement rule
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *
 */
public class ArcRefinement extends GenerationRule
{ 
	private Arc toBeRefined;


	public ArcRefinement(Petrinet net, PetrinetNode initialNodeOfNet, PetrinetNode finalNodeOfNet, ElementWeight elementWeight)
	{
		super(net, initialNodeOfNet, finalNodeOfNet, elementWeight);
		this.toBeRefined = null;
		
		selectElement(); 		
		
		if (!notFound)
		{
			addElementToPnet();
			removeArc();
		}
	}

	
	/**
	 * selects an arc to refine 
	 */
	protected void selectElement()
	{
		int totalWeight = elementWeight.getArcTotalWeight();
		
		if (totalWeight == 0)
		{
			notFound = true;
			return;
		}
		
		Map<Arc, Integer> arcMap = elementWeight.getArcWeight();
		int random = new Random().nextInt(totalWeight);
		int weightSoFar = 0;
		Iterator<Arc> it = arcMap.keySet().iterator();//to iterate all transitions in the key set

		while(it.hasNext())
		{
			Arc f = it.next();
			weightSoFar += arcMap.get(f);
			
			if (weightSoFar > random)
			{
				toBeRefined = f;
				break;
			}
		}
	}
	
	
	/**
	 * refines the selected arc and adding one place and one transition and arcs to the net 
	 */
	protected void addElementToPnet()
	{	
		PetrinetNode sourceNode = toBeRefined.getSource();
		PetrinetNode targetNode = toBeRefined.getTarget();
		
		Place p = pnet.addPlace("p" + pnet.getPlaces().size());
		elementWeight.setWeight(p, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
		
		Transition t = pnet.addTransition("t" + pnet.getTransitions().size());
		elementWeight.setWeight(t, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
		
		if (sourceNode instanceof Place)
		{
			Arc arc1 = pnet.addArc((Place)sourceNode, t);
			elementWeight.setWeight(arc1, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
			
			Arc arc2 = pnet.addArc(t, p);
			elementWeight.setWeight(arc2, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
			
			Arc arc3 = pnet.addArc(p, (Transition)targetNode);
			elementWeight.setWeight(arc3, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
			
		}
		else
		{
			Arc arc1 = pnet.addArc((Transition)sourceNode, p);
			elementWeight.setWeight(arc1, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
			
			Arc arc2 = pnet.addArc(p, t);
			elementWeight.setWeight(arc2, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
			
			Arc arc3 = pnet.addArc(t, (Place)targetNode);
			elementWeight.setWeight(arc3, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
		}
	}
	
	private void removeArc()
	{
		elementWeight.removeElementWeight(toBeRefined);
		pnet.removeEdge(toBeRefined);
	}
}
