package org.processmining.plugins.testbed.generationrule;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.plugins.testbed.util.ElementWeight;

/**
 * The class provides a template for all the construction rules 
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *
 */
abstract class GenerationRule 
{
	protected Petrinet pnet;
	protected PetrinetNode initialNodeOfNet;
	protected PetrinetNode finalNodeOfNet;
	protected ElementWeight elementWeight;
	protected boolean notFound;
	//protected LabelTable labelTable;


	public GenerationRule(Petrinet pnet, PetrinetNode initialNodeOfNet, PetrinetNode finalNodeOfNet, ElementWeight elementWeight)
	{
		this.pnet = pnet;
		this.initialNodeOfNet = initialNodeOfNet;
		this.finalNodeOfNet = finalNodeOfNet;
		this.elementWeight = elementWeight;
		this.notFound = false;
	}
	
	abstract protected void selectElement();
	
	abstract protected void addElementToPnet();
	
	/**
	 * returns whether the nodes/arcs on which a construction rule can be applied are found 
	 *  
	 * @return noFound, found or not 
	 */
	public boolean getIsFound()
	{
		return notFound;
	}
	
	/**
	 * returns the transformed Petri net by a construction rule 
	 * 
	 * @return pnet, the transformed Petri net
	 */
	public Petrinet getPnet()
	{
		return pnet;
	}
	
	
	/**
	 * returns the weight of a node/arc 
	 * 
	 * @return elementWeight, weight value of a node/arc
	 */
	public ElementWeight getElementWeight()
	{
		return elementWeight;
	}
}

