package org.processmining.plugins.testbed.generationrule;

import java.util.Iterator;
import java.util.Map;
import java.util.Random;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.testbed.util.ElementWeight;



/**
 * The class implements the Loop Addition rule
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *
 */
public class LoopAddition extends GenerationRule
{
	private Place toBeLooped;

	
	public LoopAddition(Petrinet net, PetrinetNode initialNodeOfNet, PetrinetNode finalNodeOfNet, ElementWeight elementWeight)
	{
		super(net, initialNodeOfNet, finalNodeOfNet, elementWeight);
		this.toBeLooped = null;
		
		selectElement();
		
		if (!notFound)
		{	
			addElementToPnet();
		}
	}
	

	/**
	 * selects a place to apply the rule 
	 */
	protected void selectElement()
	{
		int weight1 = 0;
		int weight2 = 0;
		boolean changed1 = false;
		boolean changed2 = false;

		if (initialNodeOfNet != null && initialNodeOfNet instanceof Place)
		{
			weight1 = elementWeight.getElementWeight((Place)initialNodeOfNet);
			elementWeight.changeElementWeight((Place)initialNodeOfNet, 0);
			changed1 = true;
		}
		
		if (finalNodeOfNet != null && finalNodeOfNet instanceof Place)
		{
			weight2 = elementWeight.getElementWeight((Place)finalNodeOfNet);
			elementWeight.changeElementWeight((Place)finalNodeOfNet, 0);
			changed2 = true;
		}
		
		int totalWeight = elementWeight.getPlaceTotalWeight();		
		
		if (totalWeight == 0)
		{
			notFound = true;
			
			if (changed1)
			{
				elementWeight.changeElementWeight((Place)initialNodeOfNet, weight1);
			}
			
			if (changed2)
			{
				elementWeight.changeElementWeight((Place)finalNodeOfNet, weight2);
			}
			
			return;			
		}
			
		
		Map<Place, Integer> placeMap = elementWeight.getPlaceWeight();//get weights of all the transitions in the net
		int random = new Random().nextInt(totalWeight);
		int weightSoFar = 0;
		Iterator<Place> it = placeMap.keySet().iterator();//to iterate all transitions in the key set

		while(it.hasNext())
		{
			Place p = it.next();
			weightSoFar += placeMap.get(p);
			
			if (weightSoFar > random)
			{
				toBeLooped = p;
				break;
			}
		}
		
		if (changed1)
		{
			elementWeight.changeElementWeight((Place)initialNodeOfNet, weight1);
		}
		
		if (changed2)
		{
			elementWeight.changeElementWeight((Place)finalNodeOfNet, weight2);
		}
		
//		if (toBeLooped == null)
//		{
//			notFound = true;
//		}
	}

	
	/**
	 * adds a transition and arcs to the net 
	 */
	protected void addElementToPnet()
	{
		Transition t = pnet.addTransition("t" + (pnet.getTransitions().size()));
		elementWeight.setWeight(t, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeLooped)));
		
		Arc arc1 = pnet.addArc(toBeLooped, t);
		elementWeight.setWeight(arc1, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeLooped)));
		
		Arc arc2 = pnet.addArc(t, toBeLooped);
		elementWeight.setWeight(arc2, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeLooped)));
		
		elementWeight.changeElementWeight(toBeLooped);
	}
}
