
package org.processmining.plugins.testbed.generationrule;

import java.util.Iterator;
import java.util.Map;
import java.util.Random;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.testbed.util.ElementWeight;


/**
 * the class implements the Transition Bridge rule
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 * 
 */
public class PlaceBridge extends GenerationRule
{
	private Transition sourceTrans; 
	private Transition targetTrans;


	public PlaceBridge(Petrinet net, PetrinetNode initialNodeOfNet, PetrinetNode finalNodeOfNet, ElementWeight elementWeight)
	{
		super(net, initialNodeOfNet, finalNodeOfNet, elementWeight);
		this.sourceTrans = null;
		this.targetTrans = null;
		
		selectElement(); 
		
		if (!notFound)
		{
			addElementToPnet();
		}
	}

	
	/**
	 * selects a pair of transitions to apply the rule
	 */
	protected void selectElement()
	{
		int weight1 = 0;
		int weight2 = 0;
		boolean changed1 = false;
		boolean changed2 = false;
		
		if (finalNodeOfNet != null && finalNodeOfNet instanceof Transition)
		{
			weight2 = elementWeight.getElementWeight((Transition)finalNodeOfNet);
			elementWeight.changeElementWeight((Transition)finalNodeOfNet, 0);
			changed2 = true;
		}
		
		int totalWeight= elementWeight.getTransitionTotalWeight();		
		
		if (totalWeight == 0)
		{
			notFound = true;
			
			if (changed2)
			{
				elementWeight.changeElementWeight((Transition)finalNodeOfNet, weight2);
			}
			
			return;
		}
		
		
		Map<Transition, Integer> transMap = elementWeight.getTransitionWeight();
		int random = new Random().nextInt(totalWeight);
		int weightSoFar = 0;
		Iterator<Transition> it = transMap.keySet().iterator();

		while(it.hasNext())
		{
			Transition t = it.next();
			weightSoFar += transMap.get(t);
			
			if (weightSoFar > random)
			{
				sourceTrans = t;
				break;
			}
		}
		
		if (changed2)
		{
			elementWeight.changeElementWeight((Transition)finalNodeOfNet, weight2);
		}
		
//		if (sourceTrans == null)
//		{
//			notFound = true;
//			return;
//		}	
//		int sourceNodeWeight = elementWeight.getElementWeight(sourceTrans);
//		elementWeight.changeElementWeight(sourceTrans, 0);
		
		if (initialNodeOfNet != null && initialNodeOfNet instanceof Transition)
		{
			weight1 = elementWeight.getElementWeight((Transition)initialNodeOfNet);
			elementWeight.changeElementWeight((Transition)initialNodeOfNet, 0);
			changed1 = true;
		}
		
		totalWeight = elementWeight.getTransitionTotalWeight();
		
		if (totalWeight == 0)
		{
			notFound = true;
			
			if (changed1)
			{
				elementWeight.changeElementWeight((Transition)initialNodeOfNet, weight1);
			}
			//elementWeight.changeElementWeight(sourceTrans, sourceNodeWeight);
			return;			
		}
			
		it = transMap.keySet().iterator();
		random = new Random().nextInt(totalWeight);
		weightSoFar = 0;

		while(it.hasNext())
		{
			Transition t = it.next();
			weightSoFar += transMap.get(t);
			
			if (weightSoFar > random)
			{
				targetTrans = t;
				break;
			}
		}

		
		if (changed1)
		{
			elementWeight.changeElementWeight((Transition)initialNodeOfNet, weight1);
		}
		
//		elementWeight.changeElementWeight(sourceTrans,sourceNodeWeight);
//		if (targetTrans == null)
//		{
//			notFound = true;
//			return;
//		}
	}
	
	/**
	 * adds a place and arcs to the net
	 */
	protected void addElementToPnet()
	{
		Place p = pnet.addPlace("p" + (pnet.getPlaces().size()));
		elementWeight.setWeight(p, elementWeight.addElementWeight(elementWeight.getElementWeight(sourceTrans), elementWeight.getElementWeight(targetTrans)));
		Arc arc1 = pnet.addArc(sourceTrans, p);
		elementWeight.setWeight(arc1, elementWeight.addElementWeight(elementWeight.getElementWeight(sourceTrans), elementWeight.getElementWeight(targetTrans)));
		Arc arc2 = pnet.addArc(p, targetTrans);
		elementWeight.setWeight(arc2, elementWeight.addElementWeight(elementWeight.getElementWeight(sourceTrans), elementWeight.getElementWeight(targetTrans)));
		elementWeight.changeElementWeight(sourceTrans);
		elementWeight.changeElementWeight(targetTrans);
	}
}