package org.processmining.plugins.testbed.generationrule;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;
import java.util.Vector;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.testbed.util.ElementWeight;


/**
 * the class implements the Place Duplication rule
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *  
 */
public class PlaceDuplication extends GenerationRule
{
	private Place toBeDuplicated;

	
	public PlaceDuplication(Petrinet net, PetrinetNode initialNodeOfNet, PetrinetNode finalNodeOfNet, ElementWeight elementWeight)
	{
		super(net, initialNodeOfNet, finalNodeOfNet, elementWeight);
		toBeDuplicated = null;
		
		selectElement(); 
		
		if (!notFound)
		{
			addElementToPnet();
		}
	}
	
	
	/**
	 * selects a place to apply the rule
	 */
	protected void selectElement()
	{
		int weight1 = 0;
		int weight2 = 0;
		boolean changed1 = false;
		boolean changed2 = false;
	
		
		if (initialNodeOfNet != null && initialNodeOfNet instanceof Place)
		{
			weight1 = elementWeight.getElementWeight((Place)initialNodeOfNet);
			elementWeight.changeElementWeight((Place)initialNodeOfNet, 0);
			changed1 = true;
		}
		
		if (finalNodeOfNet != null && finalNodeOfNet instanceof Place)
		{
			weight2 = elementWeight.getElementWeight((Place)finalNodeOfNet);
			elementWeight.changeElementWeight((Place)finalNodeOfNet, 0);
			changed2 = true;
		}
		
		int totalWeight = elementWeight.getPlaceTotalWeight();	
		
		if (totalWeight == 0)
		{
			notFound = true;
			
			if (changed1)
			{
				elementWeight.changeElementWeight((Place)initialNodeOfNet, weight1);
			}
			
			if (changed2)
			{
				elementWeight.changeElementWeight((Place)finalNodeOfNet, weight2);
			}
			
			return;
		}
			
		
		Map<Place, Integer> placeMap = elementWeight.getPlaceWeight();//get weights of all the transitions in the net
		int random = new Random().nextInt(totalWeight);
		int weightSoFar = 0;
		Iterator<Place> it = placeMap.keySet().iterator();//to iterate all transitions in the key set

		while(it.hasNext())
		{
			Place p = it.next();
			weightSoFar += placeMap.get(p);
			
			if (weightSoFar > random)
			{
				toBeDuplicated = p;
				break;
			}
		}
		
		
		if (changed1)
		{
			elementWeight.changeElementWeight((Place)initialNodeOfNet, weight1);
		}
		
		if (changed2)
		{
			elementWeight.changeElementWeight((Place)finalNodeOfNet, weight2);
		}
		
//		if (toBeDuplicated == null)
//		{
//			notFound = true;
//		}
	}

	
	/**
	 * adds a place and arcs to the net 
	 */
	protected void addElementToPnet()
	{
		Collection<Transition> sourceTransitions = new Vector<Transition>();
		Collection<Transition> targetTransitions = new Vector<Transition>();
		
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pnet.getInEdges(toBeDuplicated))
		{
			sourceTransitions.add((Transition)e.getSource());
		}
		
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pnet.getOutEdges(toBeDuplicated))
		{
			targetTransitions.add((Transition)e.getTarget());
		}
		
		Place p = pnet.addPlace("p" + (pnet.getPlaces().size()));
		elementWeight.setWeight(p, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeDuplicated)));
		elementWeight.changeElementWeight(toBeDuplicated);
		
		for (Transition sourceTrans : sourceTransitions)
		{
			Arc arc = pnet.addArc(sourceTrans, p);
			elementWeight.setWeight(arc, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeDuplicated)));
		}
		
		for (Transition targetTrans : targetTransitions)
		{
			Arc arc = pnet.addArc(p, targetTrans);
			elementWeight.setWeight(arc, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeDuplicated)));
		}
	}	
}

