package org.processmining.plugins.testbed.generationrule;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;
import java.util.Vector;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.testbed.util.ElementWeight;


/**
 * the class implements the Transition Duplication rule 
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 * 
 */
public class TransitionDuplication extends GenerationRule
{
	private Transition toBeDuplicated;
	
	
	
	public TransitionDuplication(Petrinet net, PetrinetNode initialNodeOfNet, PetrinetNode finalNodeOfNet, ElementWeight elementWeight)
	{
		super(net, initialNodeOfNet, finalNodeOfNet, elementWeight);
		toBeDuplicated = null;
		
		selectElement();
		
		if (!notFound)
		{
			addElementToPnet();
		}
	}
	
	
	/**
	 * selects a transition to apply the rule
	 */
	protected void selectElement()
	{
		int weight1 = 0;
		int weight2 = 0;
		boolean changed1 = false;
		boolean changed2 = false;
		
		if (initialNodeOfNet != null && initialNodeOfNet instanceof Transition)
		{
			weight1 = elementWeight.getElementWeight((Transition)initialNodeOfNet);
			elementWeight.changeElementWeight((Transition)initialNodeOfNet, 0);
			changed1 = true;
		}
		
		if (finalNodeOfNet != null && finalNodeOfNet instanceof Transition)
		{
			weight2 = elementWeight.getElementWeight((Transition)finalNodeOfNet);
			elementWeight.changeElementWeight((Transition)finalNodeOfNet, 0);
			changed2 = true;
		}
		
		int totalWeight = elementWeight.getTransitionTotalWeight();
		
		if (totalWeight == 0)
		{
			notFound = true;
			
			if (changed1)
			{
				elementWeight.changeElementWeight((Transition)initialNodeOfNet, weight1);
			}
			
			if (changed2)
			{
				elementWeight.changeElementWeight((Transition)finalNodeOfNet, weight2);
			}
			
			return;
		}
		
		Map<Transition, Integer> transMap = elementWeight.getTransitionWeight();
		int random = new Random().nextInt(totalWeight);
		int weightSoFar = 0;
		Iterator<Transition> it = transMap.keySet().iterator();

		while(it.hasNext())
		{
			Transition t = it.next();
			weightSoFar += transMap.get(t);
			
			if (weightSoFar > random)
			{
				toBeDuplicated = t;
				break;
			}
		}
		
		if (changed1)
		{
			elementWeight.changeElementWeight((Transition)initialNodeOfNet, weight1);
		}
		
		if (changed2)
		{
			elementWeight.changeElementWeight((Transition)finalNodeOfNet, weight2);
		}
		
//		if (toBeDuplicated == null)
//		{
//			notFound = true;
//		}
	}
	
	
	/**
	 * adds a transition and arcs to the net
	 */
	protected void addElementToPnet()
	{
		Collection<Place> inputPlaces = new Vector<Place>();
		Collection<Place> outputPlaces = new Vector<Place>();
		
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pnet.getInEdges(toBeDuplicated))
		{
			inputPlaces.add((Place)e.getSource());
		}
		
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pnet.getOutEdges(toBeDuplicated))
		{
			outputPlaces.add((Place)e.getTarget());
		}
		
		Transition t = pnet.addTransition("t" + (pnet.getTransitions().size()));
		elementWeight.setWeight(t, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeDuplicated)));

		elementWeight.changeElementWeight(toBeDuplicated);
		
		for (Place sourcePlace : inputPlaces)
		{
			Arc arc = pnet.addArc(sourcePlace, t);
			elementWeight.setWeight(arc, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeDuplicated)));
		}
		
		for (Place targetPlace : outputPlaces)
		{
			Arc arc = pnet.addArc(t, targetPlace);	
			elementWeight.setWeight(arc, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeDuplicated)));
		}	
	}
}
