package org.processmining.plugins.testbed.initialnet;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.plugins.testbed.util.ElementWeight;



/**
 * The class provides an initial Petri net with only one place for the net generation  
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *
 */
public class DefaultInitialPlaceNet 
{
	private Petrinet pnet;
	private ElementWeight elementWeight;

	public DefaultInitialPlaceNet(Petrinet pnet, ElementWeight elementWeight)
	{
		this.pnet = pnet;
		this.elementWeight = elementWeight;
		addPlace();
	}

	private void addPlace()
	{
		Place p = pnet.addPlace("p0");
		elementWeight.setWeight(p, 100); 
	}
	
	
	/**
	 * gets the initial net
	 * 
	 * @return pnet, the initial net
	 */
	public Petrinet getPnet()
	{
		return pnet;
	}
	
	
	/**
	 * gets element weight
	 * 
	 * @return elementWeight, element weight
	 */
	public ElementWeight getElementWeight()
	{
		return elementWeight;
	}
}
