package org.processmining.plugins.testbed.net;



import java.util.Vector;
import java.util.concurrent.BlockingQueue;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.testbed.result.PetriNetCollection;
import org.processmining.plugins.testbed.util.DistInitMarking;
import org.processmining.plugins.testbed.util.LabelledPnet;
import org.processmining.plugins.testbed.util.PetrinetWithInitMarking;
import org.processmining.plugins.testbed.weighttable.RuleTable;


import cern.jet.random.Poisson;
import cern.jet.random.engine.RandomEngine;


/**
 * the class invokes the net generator to generate the specified number of workflow nets and exports the nets into an external folder 
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *
 */
public class Generator extends Thread 
{ 
	//private PluginContext context;
	private RuleTable ruleTable;  
	private int meanIterationNum;
	private int netNum;
	private Vector<Integer> selected;
	private Vector<BlockingQueue<LabelledPnet>> netQueuesForTest;
	private PetriNetCollection pnetCollection;



	public Generator(RuleTable ruleTable, int netNum, int meanIterationNum, Vector<Integer> selected, Vector<BlockingQueue<LabelledPnet>> netQueuesForTest)
	{
		//this.context = context;
		this.ruleTable = ruleTable;
		this.netNum = netNum;
		this.meanIterationNum = meanIterationNum;
	    this.selected = selected;
		this.netQueuesForTest = netQueuesForTest;
		this.pnetCollection = new PetriNetCollection();
		
	    start();
	}
	
	
	
	public void run()
	{	
		//createOrEmptyFolder();
		for (int i = 0; i < this.netNum; i++)
		{
			NetGenerator netGenerator = new NetGenerator(ruleTable, calIterationNum(), Integer.toString(i + 1));
			Petrinet generatedPnet = netGenerator.getGeneratedNet();
			Marking marking = new DistInitMarking(generatedPnet).getInitialMarking();
			PetrinetWithInitMarking pnetWithMarking = new PetrinetWithInitMarking();
			pnetWithMarking.setPnet(generatedPnet);
			pnetWithMarking.setInitialMarking(marking);
			pnetCollection.addNet(pnetWithMarking);
			//pnetCollection.addNet(netGenerator.getGeneratedNet());
			//outputGeneratedNetsToFile(netGenerator.getGeneratedNet());
			LabelledPnet labelledPnet = new LabelledPnet();
			labelledPnet.setPnet(netGenerator.getGeneratedNet());
			putGeneratedPnetToStatistics(labelledPnet);
		}
		
//		createOrEmptyFolder();
//		
//		for (int i = 0; i < this.netNum; i++)
//		{
//			NetGenerator netGenerator = new NetGenerator(ruleTable, calIterationNum(), Integer.toString(i + 1));
//			outputGeneratedNetsToFile(netGenerator.getGeneratedNet());
//			LabelledPnet labelledPnet = new LabelledPnet();
//			labelledPnet.setPnet(netGenerator.getGeneratedNet());
//			putGeneratedPnetToStatistics(labelledPnet);
//		}
	}

	
//	private void createOrEmptyFolder()
//	{
//		File directory = new File("generated-Petri-nets");
//		boolean isInExistence = directory.exists();
//		
//		if (!isInExistence)
//		{
//			boolean done = directory.mkdir();
//			if (!done) { JOptionPane.showMessageDialog(null, "folder to hold the generated nets cannot be created", "Message", JOptionPane.WARNING_MESSAGE); }
//		}
//		else
//		{	
//			File[] files = directory.listFiles();
//			for (File file : files) 
//			{    
//				boolean done = file.delete();
//				if (!done) { JOptionPane.showMessageDialog(null, "any old nets in the folder cannot be deleted", "Message", JOptionPane.WARNING_MESSAGE); }	 
//			}
//		}
//	}
	
	
	private synchronized int calIterationNum()
	{
		int i = 0;
		do { i = new Poisson(meanIterationNum, RandomEngine.makeDefault()).nextInt(); } while(i == 0);
		return i;
	}


	/**
	 * returns a collection of generated Petri nets
	 * 
	 * returns pnetCollection, a collection of Petri nets
	 */
	public PetriNetCollection getCollection()
	{	
		return pnetCollection;
	}
	
	
	
//	private synchronized void outputGeneratedNetsToFile(Petrinet generatedNet)
//	{
//		PnmlExport export = new PnmlExport();
//		try { export.exportPetriNet(context, generatedNet, new File("generated-Petri-nets\\" + generatedNet.getLabel() + ".pnml")); } 
//		catch (IOException e) { e.printStackTrace(); }
//		
////		PnmlExportNetToPNML export = new PnmlExportNetToPNML();
////		try { export.exportPetriNetToPNMLFile((UIPluginContext) context, generatedNet, new File("generated-Petri-nets\\" + generatedNet.getLabel() + ".pnml")); } 
////		catch (IOException e) { e.printStackTrace(); }
//	}
	
	
	private synchronized void putGeneratedPnetToStatistics(LabelledPnet labelledPnet)
	{	
		for (int i : selected) 
		{
			try { netQueuesForTest.get(i).put(labelledPnet); } 
			catch (InterruptedException e) { e.printStackTrace(); }
		}
	}
}
	
