package org.processmining.plugins.testbed.net;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.impl.PetrinetFactory;
import org.processmining.models.graphbased.directed.petrinet.impl.PetrinetImpl;
import org.processmining.plugins.testbed.generationrule.ArcBridge;
import org.processmining.plugins.testbed.generationrule.ArcRefinement;
import org.processmining.plugins.testbed.generationrule.LoopAddition;
import org.processmining.plugins.testbed.generationrule.PlaceBridge;
import org.processmining.plugins.testbed.generationrule.PlaceDuplication;
import org.processmining.plugins.testbed.generationrule.PlaceRefinement;
import org.processmining.plugins.testbed.generationrule.TransitionBridge;
import org.processmining.plugins.testbed.generationrule.TransitionDuplication;
import org.processmining.plugins.testbed.generationrule.TransitionRefinement;
import org.processmining.plugins.testbed.initialnet.DefaultInitialPlaceNet;
import org.processmining.plugins.testbed.util.ElementWeight;
import org.processmining.plugins.testbed.util.Rule;
import org.processmining.plugins.testbed.weighttable.RuleTable;


/**
 * the class generates workflow nets using the construction rules starting with the initial net 
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *
 */
public class NetGenerator 
{	
	private RuleTable ruleTable;
	private Petrinet generatedNet;
	private ElementWeight elementWeight;
	private int iterationNum;

	
	public NetGenerator(RuleTable ruleTable, int iterationNum, String netName)
	{
		this.ruleTable = ruleTable;
		this.iterationNum = iterationNum;
		this.generatedNet = PetrinetFactory.newPetrinet("net" + netName); 
		this.elementWeight = new ElementWeight();
		generateNet();
	}
	
	
	/**
	 * returns the generated net
	 * 
	 * @return generatedNet, the net generated
	 */
	public Petrinet getGeneratedNet()
	{
		return generatedNet;
	}


	private void generateNet()
	{
		int i = 0;
		PetrinetNode initialNodeOfNet = null;
		PetrinetNode finalNodeOfNet = null;
		int totalRuleWeight = ruleTable.getTotalRuleWeight();

		DefaultInitialPlaceNet defaultInitialPlacenet = new DefaultInitialPlaceNet(generatedNet, elementWeight);//, elementLabel);
		generatedNet = defaultInitialPlacenet.getPnet();
		elementWeight = defaultInitialPlacenet.getElementWeight();
		//elementLabel = defaultInitialPlacenet.getElementLabel();
			
		PlaceRefinement rule = new PlaceRefinement(generatedNet, initialNodeOfNet, finalNodeOfNet, elementWeight);//, elementLabel);	
		generatedNet = rule.getPnet();
		elementWeight = rule.getElementWeight();
		//elementLabel = rule.getElementLabel();
			
		for (Place p : generatedNet.getPlaces())
		{
			if (generatedNet.getInEdges(p).size() == 0 && generatedNet.getOutEdges(p).size() != 0) { initialNodeOfNet = p; }
			if (generatedNet.getOutEdges(p).size() == 0 && generatedNet.getInEdges(p).size() != 0) { finalNodeOfNet = p; }
		}

		
		while (i < iterationNum - 1) 
		{
			int rId = ruleTable.getRuleId(totalRuleWeight);

			if (rId == Rule.PLACEREFINEMENT.getIndex())
			{				
				PlaceRefinement r = new PlaceRefinement(generatedNet, initialNodeOfNet, finalNodeOfNet, elementWeight);//, elementLabel);
				
				if (!r.getIsFound())
				{
					generatedNet = r.getPnet();
					elementWeight = r.getElementWeight();
					//elementLabel = r.getElementLabel();
					i++;
				}
			}
			else if (rId == Rule.TRANSITIONREFINEMENT.getIndex())
			{
				TransitionRefinement r = new TransitionRefinement(generatedNet, initialNodeOfNet, finalNodeOfNet, elementWeight);//, elementLabel);
				
				if (!r.getIsFound())
				{
					generatedNet = r.getPnet();
					elementWeight = r.getElementWeight();
					//elementLabel = r.getElementLabel();
					i++;
				}
			}
			else if (rId == Rule.ARCREFINEMENT.getIndex())	
			{
				ArcRefinement r = new ArcRefinement(generatedNet, initialNodeOfNet, finalNodeOfNet, elementWeight);//, elementLabel);
				
				if (!r.getIsFound())
				{
					generatedNet = r.getPnet();
					elementWeight = r.getElementWeight();
					//elementLabel = r.getElementLabel();
					i++;
				}
			}
			else if (rId == Rule.PLACEDUPLICATION.getIndex())
			{
				PlaceDuplication r = new PlaceDuplication(generatedNet, initialNodeOfNet, finalNodeOfNet, elementWeight);//, elementLabel);
		
				if (!r.getIsFound())
				{
					generatedNet = r.getPnet();
					elementWeight = r.getElementWeight();
					//elementLabel = r.getElementLabel();
					i++;
				}
			}
			else if (rId == Rule.TRANSITIONDUPLICATION.getIndex())
			{
				TransitionDuplication r = new TransitionDuplication(generatedNet, initialNodeOfNet, finalNodeOfNet, elementWeight);//, elementLabel);
				
				if (!r.getIsFound())
				{
					generatedNet = r.getPnet();
					elementWeight = r.getElementWeight();
					//elementLabel = r.getElementLabel();
					i++;
				}
			}
			else if (rId == Rule.LOOPADDITION.getIndex())
			{
				LoopAddition r = new LoopAddition(generatedNet, initialNodeOfNet, finalNodeOfNet, elementWeight);//, elementLabel);
				
				if (!r.getIsFound())
				{
					generatedNet = r.getPnet();
					elementWeight = r.getElementWeight();
					//elementLabel = r.getElementLabel();
					i++;
				}
			}
			else if (rId == Rule.PLACEBRIDGE.getIndex())
			{
				PlaceBridge r = new PlaceBridge(generatedNet, initialNodeOfNet, finalNodeOfNet, elementWeight);//, elementLabel);
				
				if (!r.getIsFound())
				{
					generatedNet = r.getPnet();
					elementWeight = r.getElementWeight();
					//elementLabel = r.getElementLabel();
					i++;
				}
			}
			else if (rId == Rule.ARCBRIDGE.getIndex())
			{
				ArcBridge r = new ArcBridge(generatedNet, initialNodeOfNet, finalNodeOfNet, elementWeight);//, elementLabel);
				
				if (!r.getIsFound())
				{
					generatedNet = r.getPnet();
					elementWeight = r.getElementWeight();
					//elementLabel = r.getElementLabel();
					i++;
				}
			}
			else
			{
				TransitionBridge r = new TransitionBridge(generatedNet, initialNodeOfNet, finalNodeOfNet, elementWeight);//, elementLabel);
				
				if (!r.getIsFound())
				{
					generatedNet = r.getPnet();
					elementWeight = r.getElementWeight();
					//elementLabel = r.getElementLabel();
					i++;
				}
			}
		}
	}
}
