package org.processmining.plugins.testbed.reporter;

import java.util.Vector;
import java.util.concurrent.BlockingQueue;

import org.processmining.plugins.testbed.result.Report;
import org.processmining.plugins.testbed.result.ReportTableSetting;
import org.processmining.plugins.testbed.result.Results;
import org.processmining.plugins.testbed.util.Test;


/**
 * the class aggregates the results of all the experiments  
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *
 */
public class ReportAggregator 
{
	private Vector<Integer> selected;
	private Results testReport;
	private int xthPercentile;
	private BlockingQueue<Vector<Report>> pathReport;
	private BlockingQueue<Vector<Report>> nodeNumReport;
	private BlockingQueue<Vector<Report>> faninFanoutReport;
	private BlockingQueue<Vector<Report>> findingPairReport;
	private BlockingQueue<Vector<Report>> soundnessReport;

	
	public ReportAggregator(int netNum, Vector<Integer> selected, int xthPercentile)
	{
		this.selected = selected;
		this.testReport = new Results();
		this.xthPercentile = xthPercentile;
	}
	
	
	/**
	 * sets the shortest path result 
	 * 
	 * @param pathReport, the length of a shortest path
	 */
	public void setPathReport(BlockingQueue<Vector<Report>> pathReport)
	{
		this.pathReport = pathReport;
	}
	
	
	/**
	 * sets the number of nodes result
	 * 
	 * @param nodeNumReport, number of places and transitions
	 */
	public void setNodeNumReport(BlockingQueue<Vector<Report>> nodeNumReport)
	{
		this.nodeNumReport = nodeNumReport;
	}
	
	
	/**
	 * sets the average fanin/fanout result
	 * 
	 * @param faninFanoutReport, the average fanin and fanout of nodes 
	 */
	public void setFaninFanoutReport(BlockingQueue<Vector<Report>> faninFanoutReport)
	{
		this.faninFanoutReport = faninFanoutReport;
	}
	
	
	/**
	 * sets the causal pair finding result
	 *  
	 * @param findingPairReport, coverage of causal pairs 
	 */
	public void setFindingPairReport(BlockingQueue<Vector<Report>> findingPairReport)
	{
		this.findingPairReport = findingPairReport;
	}
	
	
	/**
	 * sets the soundness result
	 *  
	 * @param soundnessReport, soundness 
	 */
	public void setSoundnessReport(BlockingQueue<Vector<Report>> soundnessReport)
	{
		this.soundnessReport = soundnessReport;
	}
	
	
	/**
	 * returns the report of all the experimental results
	 *  
	 * @return testReport, all the experimental results 
	 * @throws InterruptedException
	 */
	public Results getReport() throws InterruptedException
	{

		for (int i : selected)
		{
			if (i == Test.PATHLENGTH.getIndex())
			{
				testReport.addTestReport(pathReport.take().get(0));
			}
			else if (i == Test.NUMOFNODES.getIndex())
			{
				Vector<Report> r = nodeNumReport.take();
				
				testReport.addTestReport(r.get(0));
				testReport.addTestReport(r.get(1));
			}
			else if (i == Test.FANINFANOUT.getIndex())
			{			
				Vector<Report> r = faninFanoutReport.take();
				
				testReport.addTestReport(r.get(0));
				testReport.addTestReport(r.get(1));
				testReport.addTestReport(r.get(2));
				testReport.addTestReport(r.get(3));
			}
			else if (i == Test.PAIRFINDING.getIndex())
			{
				Vector<Report> r = findingPairReport.take();
				
				testReport.addTestReport(r.get(0));
				testReport.addTestReport(r.get(1));
				
			}
			else
			{
				Vector<Report> r = soundnessReport.take();
				
				testReport.addTestReport(r.get(0));
			}
		}
		
		ReportTableSetting setting = new ReportTableSetting();
		setting.setPercentile(xthPercentile);
		testReport.setSetting(setting);
		return testReport;
	}
}
