package org.processmining.plugins.testbed.tester;

import java.util.Vector;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.plugins.testbed.experiment.BFSPath;
import org.processmining.plugins.testbed.experiment.FaninFanout;
import org.processmining.plugins.testbed.experiment.FindingPair;
import org.processmining.plugins.testbed.experiment.NodeNum;
import org.processmining.plugins.testbed.experiment.Soundness;
import org.processmining.plugins.testbed.net.Generator;
import org.processmining.plugins.testbed.net.Loader;
import org.processmining.plugins.testbed.reporter.ReportAggregator;
import org.processmining.plugins.testbed.result.Report;
import org.processmining.plugins.testbed.result.Results;
import org.processmining.plugins.testbed.util.LabelledPnet;
import org.processmining.plugins.testbed.util.Test;
import org.processmining.plugins.testbed.weighttable.RuleTable;



/**
 * the class invokes the net generator to generate the specified number of workflow nets and exports the nets into a resource 
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *
 */
public class Tester 
{
	private RuleTable ruleTable;
	private int netNum;
	private int avgiterationNum;
	private int testPerNet;
	private int xthPercentile;
	private PluginContext context;
	private Vector<Integer> selected;
	private boolean loaded;
	private Petrinet loadedPnet;
	private Generator generator; 
	
	
	
	private Vector<BlockingQueue<LabelledPnet>> netQueuesForTest;
//	private BlockingQueue<LabelledPnet> shortestPathQueue;
//	private BlockingQueue<LabelledPnet> numOfNodeQueue;
//	private BlockingQueue<LabelledPnet> faninFanoutQueue;
//	private BlockingQueue<LabelledPnet> findingLabelQueue;
//	private BlockingQueue<LabelledPnet> findingPairQueue;
//	private BlockingQueue<LabelledPnet> soundnessQueue;
	private Vector<BlockingQueue<Vector<Report>>> testReports; 
//	private BlockingQueue<Vector<Report>> shortestPathReport;
//	private BlockingQueue<Vector<Report>> nodeNumReport;
//	private BlockingQueue<Vector<Report>> faninFanoutReport;
//	private BlockingQueue<Vector<Report>> findingLabelReport;
//	private BlockingQueue<Vector<Report>> findingPairReport;
//	private BlockingQueue<Vector<Report>> soundnessReport;
	private ReportAggregator reportAggregator;


	public Tester(PluginContext context, RuleTable ruleTable, int netNum, int avgiterationNum, int testPerNet, int xthPercentile, Vector<Integer> selected)
	{
		this.context = context;
		this.ruleTable = ruleTable;
		this.netNum = netNum;
		this.avgiterationNum = avgiterationNum;
		this.testPerNet = testPerNet;
		this.xthPercentile = xthPercentile;
		this.selected = selected;
		this.loaded = false;
		
		netQueuesForTest = new Vector<BlockingQueue<LabelledPnet>>();
		for (int i = 0; i < 6; i++)
		{
			BlockingQueue<LabelledPnet> queue = new LinkedBlockingQueue<LabelledPnet>();
			netQueuesForTest.add(queue);
		}
		
		testReports = new Vector<BlockingQueue<Vector<Report>>>();
		for (int i = 0; i < 6; i++)
		{
			BlockingQueue<Vector<Report>> queue = new LinkedBlockingQueue<Vector<Report>>();
			testReports.add(queue);
		}

		workflow();
	}

	
	public Tester(PluginContext context, int netNum, int testPerNet, int xthPercentile, Vector<Integer> selected, Petrinet loadedPnet)
	{
		this.context = context;
		this.netNum = netNum;
		this.testPerNet = testPerNet;
		this.xthPercentile = xthPercentile;
		this.selected = selected;
		this.loaded = true;
		this.loadedPnet = loadedPnet;
		
		netQueuesForTest = new Vector<BlockingQueue<LabelledPnet>>();
		for (int i = 0; i < 6; i++)
		{
			BlockingQueue<LabelledPnet> queue = new LinkedBlockingQueue<LabelledPnet>();
			netQueuesForTest.add(queue);
		}

		testReports = new Vector<BlockingQueue<Vector<Report>>>();
		for (int i = 0; i < 6; i++)
		{
			BlockingQueue<Vector<Report>> queue = new LinkedBlockingQueue<Vector<Report>>();
			testReports.add(queue);
		}

		workflow();
	}
	
	
	private void workflow()
	{
		if (!loaded)
			//new Generator(context, ruleTable, netNum, avgiterationNum, selected, netQueuesForTest);
			
			
			//generator = new Generator(context, ruleTable, netNum, avgiterationNum, selected, netQueuesForTest);
		
			generator = new Generator(ruleTable, netNum, avgiterationNum, selected, netQueuesForTest);
		
		
		
		else
			new Loader(loadedPnet, netQueuesForTest, selected);
		
		reportAggregator = new ReportAggregator(netNum, selected, xthPercentile);
		
		for (int i : selected)
		{
			if (i == Test.PATHLENGTH.getIndex())
			{
				new BFSPath(netQueuesForTest.get(Test.PATHLENGTH.getIndex()), testReports.get(Test.PATHLENGTH.getIndex()), netNum, xthPercentile);
				reportAggregator.setPathReport(testReports.get(Test.PATHLENGTH.getIndex()));
			}
			else if (i == Test.NUMOFNODES.getIndex())
			{
				new NodeNum(netQueuesForTest.get(Test.NUMOFNODES.getIndex()), testReports.get(Test.NUMOFNODES.getIndex()), netNum, xthPercentile);
				reportAggregator.setNodeNumReport(testReports.get(Test.NUMOFNODES.getIndex()));
			}	
			else if (i == Test.FANINFANOUT.getIndex())
			{
				new FaninFanout(netQueuesForTest.get(Test.FANINFANOUT.getIndex()), testReports.get(Test.FANINFANOUT.getIndex()), netNum, xthPercentile);
				reportAggregator.setFaninFanoutReport(testReports.get(Test.FANINFANOUT.getIndex()));
			}
//			else if (i == Test.LABELFINDING.getIndex())
//			{
//				new FindingLabel(netQueuesForTest.get(Test.LABELFINDING.getIndex()), testReports.get(Test.LABELFINDING.getIndex()), testPerNet, netNum, bound, livelockBound, xthPercentile);
//				reportAggregator.setFindingLabelReport(testReports.get(Test.LABELFINDING.getIndex()));
//			}
			else if (i == Test.PAIRFINDING.getIndex())
			{
				new FindingPair(context, netQueuesForTest.get(Test.PAIRFINDING.getIndex()), testReports.get(Test.PAIRFINDING.getIndex()), testPerNet, netNum, xthPercentile);
				reportAggregator.setFindingPairReport(testReports.get(Test.PAIRFINDING.getIndex()));
			}
			else 
			{
				new Soundness(netQueuesForTest.get(Test.SOUNDNESS.getIndex()), testReports.get(Test.SOUNDNESS.getIndex()), netNum, context, xthPercentile);
				reportAggregator.setSoundnessReport(testReports.get(Test.SOUNDNESS.getIndex()));
			}
		}
	}
	
	
	/**
	 * returns testing report
	 * @return report 
	 * @throws InterruptedException
	 */
//	public Results getReport() throws InterruptedException
//	{
//		return reportAggregator.getReport();
//	}
	
	
	/**
	 * returns testing report and Petri nets generated
	 * @return reportAndNets, testing report and Petri ntes generated 
	 */
	public Object[] getReportAndGeneratedPnets() throws InterruptedException
	{
		Object[] reportAndNets = {reportAggregator.getReport(), generator.getCollection()};
		//System.out.println("number of nets from tester :    "  +  generator.getCollection().getGeneratedPnet().size());
		return reportAndNets;
	}
	
	
	/**
	 * returns testing report
	 * @return report 
	 * @throws InterruptedException
	 */
	public Results getReport() throws InterruptedException
	{
		return reportAggregator.getReport();
	}
}
