package org.processmining.plugins.testbed.tester;

import java.util.Vector;

import javax.swing.JPanel;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.plugins.testbed.gui.TesterPanelLite;
import org.processmining.plugins.testbed.result.Results;
import org.processmining.plugins.testbed.util.Test;

@Plugin(name = "TestBed Lite", parameterLabels = {"Net Number", "Number of Tests per Net", "Percentile", "Characteristics", "Loaded Petri Net"}, returnLabels = {"Report"}, returnTypes = {Results.class}, userAccessible = true)
public class TesterLitePlugin 
{
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "Z.Liu", email = "z.liu3@tue.nl", uiLabel=UITopiaVariant.USEPLUGIN)
	@PluginVariant(variantLabel = "User select for interaction", requiredParameterLabels = {4})
	public static Results testerUI(final UIPluginContext context, final Petrinet loadedPnet) throws InterruptedException
	{
		TesterPanelLite spanel = new TesterPanelLite();
		JPanel jpanel = spanel.getPanel();
		
		if (context.showConfiguration("Configuration of TestBed", jpanel).equals(InteractionResult.CONTINUE))
		{	
			Vector<Integer> selected = new Vector<Integer>();
			if (spanel.getCheckBox1()) { selected.add(Test.PATHLENGTH.getIndex()); }
			if (spanel.getCheckBox2()) { selected.add(Test.NUMOFNODES.getIndex()); }
			if (spanel.getCheckBox3()) { selected.add(Test.FANINFANOUT.getIndex()); }
			if (spanel.getCheckBox5()) { selected.add(Test.PAIRFINDING.getIndex()); }
			if (spanel.getCheckBox6()) { selected.add(Test.SOUNDNESS.getIndex()); }

			int netNum = 1;
			int testPerNet = spanel.getSpinner2Value();
			int xthPercentile = spanel.getSpinner1Value();	

			return tester(context, netNum, testPerNet, xthPercentile, selected, loadedPnet);
		}
		else
		{
			context.getFutureResult(0).cancel(true);
			return null;
		}
	}
	
	@PluginVariant(variantLabel = "Use default setting", requiredParameterLabels = {0, 1, 2, 3, 4})
	public static Results tester(final PluginContext context, final int netNum, final int testPerNet, 
			final int xthPercentile, final Vector<Integer> selected, Petrinet loadedPnet) throws InterruptedException
	{
		return new Tester(context, netNum, testPerNet, xthPercentile, selected, loadedPnet).getReport(); 
	}
}
