package org.processmining.plugins.testbed.tester;

import java.util.Vector;

import javax.swing.JPanel;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.plugins.testbed.gui.TesterPanel;
import org.processmining.plugins.testbed.result.PetriNetCollection;
import org.processmining.plugins.testbed.result.Results;
import org.processmining.plugins.testbed.util.Rule;
import org.processmining.plugins.testbed.util.Test;
import org.processmining.plugins.testbed.weighttable.RuleTable;




@Plugin(name = "TestBed", parameterLabels = {"Rule Table", "Number of Nets", "Net Depth", "Number of Tests per Net", "Percentile", "Characteristics"}, returnLabels = {"Characteristic Report", "Collection of Generated Petri Nets"}, returnTypes = {Results.class, PetriNetCollection.class}, userAccessible = true)
public class TesterPlugin 
{
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "Z.Liu", email = "z.liu3@tue.nl", uiLabel=UITopiaVariant.USEPLUGIN)
	@PluginVariant(variantLabel = "User select for interaction", requiredParameterLabels = {})
	public static Object[] testerUI(final UIPluginContext context) throws InterruptedException
	{
		TesterPanel spanel = new TesterPanel();
		JPanel jpanel = spanel.getPanel();
		
		if (context.showConfiguration("Configuration of TestBed", jpanel).equals(InteractionResult.CONTINUE))
		{
			RuleTable rTable = new RuleTable();
			rTable.addRule(Rule.PLACEREFINEMENT.getIndex(), spanel.getSlider1Value()); //place refinement
			rTable.addRule(Rule.TRANSITIONREFINEMENT.getIndex(), spanel.getSlider2Value()); //transition refinement
			rTable.addRule(Rule.ARCREFINEMENT.getIndex(), spanel.getSlider3Value()); //arc refinement
			rTable.addRule(Rule.PLACEBRIDGE.getIndex(), spanel.getSlider4Value()); //place bridge
			rTable.addRule(Rule.TRANSITIONBRIDGE.getIndex(), spanel.getSlider5Value()); //transition bridge
			rTable.addRule(Rule.ARCBRIDGE.getIndex(), spanel.getSlider6Value()); //arc bridge
			rTable.addRule(Rule.LOOPADDITION.getIndex(), spanel.getSlider7Value());	//loop addition
			rTable.addRule(Rule.PLACEDUPLICATION.getIndex(), spanel.getSlider8Value());	//place duplication
			rTable.addRule(Rule.TRANSITIONDUPLICATION.getIndex(), spanel.getSlider9Value());	//transition  duplication
			
			Vector<Integer> selected = new Vector<Integer>();
			if (spanel.getCheckBox1()) { selected.add(Test.PATHLENGTH.getIndex()); }
			if (spanel.getCheckBox2()) { selected.add(Test.NUMOFNODES.getIndex()); }
			if (spanel.getCheckBox3()) { selected.add(Test.FANINFANOUT.getIndex()); }
			if (spanel.getCheckBox5()) { selected.add(Test.PAIRFINDING.getIndex()); }
			if (spanel.getCheckBox6()) { selected.add(Test.SOUNDNESS.getIndex()); }
		
			int netNum = spanel.getTextField1Value();
			int avgIterationNum = spanel.getTextField2Value();
			int testperNet = spanel.getTextField4Value();
			int xthPercentile = spanel.getTextField7Value();
			
			
			return tester(context, rTable, netNum, avgIterationNum, testperNet, xthPercentile, selected);
		}
		else
		{
			context.getFutureResult(0).cancel(true);
			return null;
		}
	}
	
//	@PluginVariant(variantLabel = "Use default setting", requiredParameterLabels = {0, 1, 2, 3 ,4, 5})
//	public static Results tester(final PluginContext context, final RuleTable ruleTable, final int netNum, final int avgIterationNum, 
//			final int testperNet, final int xthPercentile, final Vector<Integer> selected) throws InterruptedException
//	{
//		return new Tester(context, ruleTable, netNum, avgIterationNum, testperNet, xthPercentile, selected).getReport(); 
//	}
	
	
	@PluginVariant(variantLabel = "Use default setting", requiredParameterLabels = {0, 1, 2, 3 ,4, 5})
	public static Object[] tester(final PluginContext context, final RuleTable ruleTable, final int netNum, final int avgIterationNum, 
			final int testperNet, final int xthPercentile, final Vector<Integer> selected) throws InterruptedException
	{
		return new Tester(context, ruleTable, netNum, avgIterationNum, testperNet, xthPercentile, selected).getReportAndGeneratedPnets(); 
	}
}


