package org.processmining.plugins.testbed.util;

import java.util.Collection;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.semantics.petrinet.Marking;


/**
 * The class distributes initial marking over a Petri net
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 16/June/2010
 *
 */
public class DistInitMarking 
{
	private Marking initialMarking;
	private Petrinet pnet;
	
	
	public DistInitMarking(Petrinet pnet)
	{
		this.pnet = pnet;
		this.initialMarking = new Marking();
		
		findMarking();
	}
	
	
	
	private void findMarking()
	{
		Collection<Place> places = pnet.getPlaces();
		
		for (Place p : places)
		{
			if (pnet.getInEdges(p).size() == 0)
			{
				initialMarking.add(p);
				break;
			}
		}
		
	}
	
	/**
	 * returns initial marking
	 * @return initialMarking, initial marking of a Petri net 
	 */
	public Marking getInitialMarking()
	{
		return initialMarking;
	}
}
