package org.processmining.plugins.testbed.visualiser;

import java.awt.Component;
import java.awt.Font;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.JTableHeader;

import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.plugins.testbed.result.Report;
import org.processmining.plugins.testbed.result.ReportTableSetting;
import org.processmining.plugins.testbed.result.Results;



/**
 * The class visualizes the characteristic report
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *
 */
public class TestbedReportVisualiser 
{
    @SuppressWarnings("serial")
    @Plugin(name = "Report visualization", returnLabels = {"Visualized Report" }, returnTypes = {JComponent.class}, parameterLabels = {"Report to visualize"}, userAccessible = false)
	@Visualizer
    public static JComponent visualize(final PluginContext context, final Results myReport)
    {
    	try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} 
		catch (Exception e) {
			e.printStackTrace();
		}

		JTable table = new JTable(0,7)
		{
			@Override
			public JTableHeader createDefaultTableHeader()
			{
				ReportTableSetting setting = myReport.getSetting();
				int xthPercentile = setting.getXthPercentile();
				int ythPercentile = setting.getYthPercentile();
				Header header = new Header(20);
				header.add(new HeaderCellLabel("characteristics ", 0, 0, 1, 1), new HeaderCellConstraints(0, 0, 1));
				header.add(new HeaderCellLabel("name of dependent plugin", 0, 0, 1, 1), new HeaderCellConstraints(1, 0, 1));
				header.add(new HeaderCellLabel("mean", 0, 0, 1, 1), new HeaderCellConstraints(2, 0, 1));
				header.add(new HeaderCellLabel("std.dev.", 0, 0, 1, 1), new HeaderCellConstraints(3, 0, 1));
				header.add(new HeaderCellLabel("median ", 0, 0, 1, 1), new HeaderCellConstraints(4, 0, 1));
				header.add(new HeaderCellLabel(xthPercentile + "th percentile", 0, 0, 1, 1), new HeaderCellConstraints(5, 0, 1));
				header.add(new HeaderCellLabel(ythPercentile + "th percentile", 0, 0, 1, 1), new HeaderCellConstraints(6, 0, 1));
			
				return header;
			}
		};

		table.setRowHeight(25);
		JScrollPane scrollPanel = new JScrollPane(table);
		table.setDefaultRenderer(Object.class, new DefaultTableCellRenderer() 
		{
		    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) 
		    {
		        Component comp = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
		        Font font = comp.getFont();
		    
		        if (column == 0) 
		        {
		            comp.setFont(font.deriveFont(Font.BOLD));
		        }
		        else 
		        {    
		            comp.setFont(font.deriveFont(Font.PLAIN));
		        }
		        return comp;
		    }
		});
	
		table.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_ALL_COLUMNS); 
        Vector<Report> allReports =  myReport.getTestReport();
		for (int i = 0; i < allReports.size(); i++)
		{		
			((DefaultTableModel)table.getModel()).addRow(allReports.get(i).getReport());	
		}
		
		return scrollPanel;
    }
}

