package org.processmining.plugins.testbed.weighttable;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Random;


/**
 * The class builds a weight table for all the rules. The table maps a rule to its probability.
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 * 
 */
public class RuleTable 
{
	private Hashtable<Integer, Integer> ruleTable;
	
	public RuleTable()
	{
		this.ruleTable = new Hashtable<Integer, Integer>();
	}
	
	/**
	 *returns the rule probability table
	 *
	 *@return ruleProbabilityTable, mapping a rule to its weight 
	 */
	public Hashtable<Integer, Integer> getRuleTable()
	{
		return ruleTable;
	}
	
	
	/**
	 * randomly selects a rule based on the probabilities 
	 * 
	 * @param totalRuleWeight, total weights of all the rules 
	 * @return ruleId, the id of the selected rule
	 */
	public int getRuleId(int totalRuleWeight)
	{
		int random = new Random().nextInt(totalRuleWeight);
		int weightSoFar = 0;
		int rId = 0;
		Enumeration<Integer> keys = ruleTable.keys();

		while(keys.hasMoreElements())
		{
			Object key = keys.nextElement();
			weightSoFar += ruleTable.get(key);
			
			if (weightSoFar > random)
			{
				rId = (Integer)key;
				break;
			}
		}
		
		return rId;
	}
	
	
	/**
	 * adds a rule to the rule table
	 * 
	 * @param id, id of the rule to be added 
	 * @param weight, weight of the rule to be added 
	 */
	public void addRule(int id, int weight)
	{
		ruleTable.put(id, weight);
	}
	
	
	/**
	 * returns the total weight of all the rules 
	 * 
	 * @return totalWeight, total rule weights 
	 */
	public int getTotalRuleWeight()
	{
		int totalWeight = 0;
		Enumeration<Integer> keys = ruleTable.keys();
		
		while (keys.hasMoreElements())
		{
			Object key = keys.nextElement();
			totalWeight += ruleTable.get(key);
		}
		
		return totalWeight;
	}
}
